from nose.plugins.skip import SkipTest


def test_ok():
    pass


def test_err():
    raise Exception("oh no")


def test_fail():
    assert False, "bye"


def test_skip():
    raise SkipTest("not me")



class Test1(object):
    def test_nested_generator(self):
        def func():
            pass

        yield func,

    def test_nested_generator_mult(self):
        def f2(a):
            pass

        for b in range(1, 4):
            yield f2, b

    def try_something(self, a):
        pass

    def test_normal_generator(self):
        yield self.try_something, 1
        yield 'try_something', 2



import unittest

from nose import case
from nose.config import Config
from nose.plugins import debug
from nose.plugins.manager import PluginManager
from nose.plugins.skip import Skip, SkipTest
from nose.proxy import ResultProxyFactory


class StubPdb:
    called = False

    def post_mortem(self, tb):
        self.called = True


class TestSkipPdbInteraction(unittest.TestCase):
    """Tests interaction between skip plugin and pdb plugin -- pdb should
    not fire on a skip error
    """

    def setUp(self):
        self._pdb = debug.pdb
        debug.pdb = StubPdb()

    def tearDown(self):
        debug.pdb = self._pdb

    def test_skip_prevents_pdb_call(self):
        class TC(unittest.TestCase):
            def test(self):
                raise SkipTest('not me')

        skip = Skip()
        skip.enabled = True
        p = debug.Pdb()
        p.enabled = True
        p.enabled_for_errors = True
        res = unittest.TestResult()
        conf = Config(plugins=PluginManager(plugins=[skip, p]))
        rpf = ResultProxyFactory(conf)
        test = case.Test(TC('test'), resultProxy=rpf)
        test(res)

        assert not res.errors, "Skip was recorded as error %s" % res.errors
        assert not debug.pdb.called, "pdb was called"


if __name__ == '__main__':
    unittest.main()



import optparse
import unittest

import nose.plugins


class OptionProcessingTests(unittest.TestCase):

    def test_enable_plugin(self):
        class NamedPlugin(nose.plugins.Plugin):
            name = "jim-bob"

        def parse_options(env, args_in):
            plugin = NamedPlugin()
            parser = optparse.OptionParser()
            plugin.options(parser, env)
            options, args = parser.parse_args(args_in)
            return options

        options = parse_options({}, [])
        assert not options.enable_plugin_jim_bob, \
            "Plugin should not be enabled"
        options = parse_options({"NOSE_WITH_JIM_BOB": "1"}, [])
        assert options.enable_plugin_jim_bob, \
            "Plugin should be enabled"
        options = parse_options({}, ["--with-jim-bob"])
        assert options.enable_plugin_jim_bob, \
            "Plugin should be enabled"
        options = parse_options({"NOSE_WITH_JIM_BOB": "1"}, ["--with-jim-bob"])
        assert options.enable_plugin_jim_bob, \
            "Plugin should be enabled"


if __name__ == '__main__':
    unittest.main()



print
"test_mod imported!"

import maths
from test_pak import state


def setup():
    print
    "MOD setup called", state, id(state)
    state.append('test_pak.test_mod.setup')


def test_add():
    print
    "MOD.test_add called", state, id(state)
    state.append('test_pak.test_mod.test_add')
    assert maths.add(1, 2) == 3


def test_minus():
    state.append('test_pak.test_mod.test_minus')


def teardown():
    print
    "MOD teardown called", state, id(state)
    state.append('test_pak.test_mod.teardown')



from nose import tools


def test_z():
    """(1) test z"""
    pass


def test_a():
    """(2) test a"""
    pass


def test_rz():
    """(3) Test with raises decorator"""
    raise TypeError("err")


test_rz = tools.raises(TypeError)(test_rz)


def decorate(func):
    func.attr = 1
    return func


def dec_replace(func):
    def newfunc():
        func()
        pass

    return newfunc


def dec_makedecorator(func):
    def newfunc():
        pass

    newfunc = tools.make_decorator(func)(newfunc)
    return newfunc


def test_dz():
    """(4) Test with non-replacing decorator"""
    pass


test_dz = decorate(test_dz)


def test_rz():
    """(5) Test with replacing decorator"""
    pass


test_rz = dec_replace(test_rz)


def test_mdz():
    """(6) Test with make_decorator decorator"""
    pass


test_mdz = dec_makedecorator(test_mdz)


def test_b():
    """(7) test b"""
    pass



import os
import unittest

from nose.plugins import PluginTester

support = os.path.join(os.path.dirname(__file__), 'support', 'issue513')


class TestPrintedTraceback(PluginTester, unittest.TestCase):
    args = ['--where=' + support]
    activate = "-v"

    def makeSuite(self):
        # make PluginTester happy, because we don't specify suitepath, we
        # have to implement this function
        return None

    def test_correct_exception_raised(self):
        print
        print
        '!' * 70
        print
        str(self.output)
        print
        '!' * 70
        print

        # Look for the line in the traceback causing the failure
        assert "raise '\\xf1'.encode('ASCII')" in str(self.output)
        assert 'FAILED (errors=1)' in str(self.output)


if __name__ == '__main__':
    unittest.main()



"""
ErrorClass Plugins
------------------

ErrorClass plugins provide an easy way to add support for custom
handling of particular classes of exceptions.

An ErrorClass plugin defines one or more ErrorClasses and how each is
handled and reported on. Each error class is stored in a different
attribute on the result, and reported separately. Each error class must
indicate the exceptions that fall under that class, the label to use
for reporting, and whether exceptions of the class should be
considered as failures for the whole test run.
"""

from nose.plugins.base import Plugin
from nose.pyversion import make_instancemethod
from nose.result import TextTestResult, NoseTextTestResult
from nose.util import isclass


class MetaErrorClass(type):
    """Metaclass for ErrorClassPlugins that allows error classes to be
    set up in a declarative manner.
    """

    def __init__(self, name, bases, attr):
        errorClasses = []
        for name, detail in attr.items():
            if isinstance(detail, ErrorClass):
                attr.pop(name)
                for cls in detail:
                    errorClasses.append(
                        (cls, (name, detail.label, detail.isfailure)))
        super(MetaErrorClass, self).__init__(name, bases, attr)
        self.errorClasses = tuple(errorClasses)


class ErrorClass(object):
    def __init__(self, *errorClasses, **kw):
        self.errorClasses = errorClasses
        try:
            for key in ('label', 'isfailure'):
                setattr(self, key, kw.pop(key))
        except KeyError:
            raise TypeError("%r is a required named argument for ErrorClass"
                            % key)

    def __iter__(self):
        return iter(self.errorClasses)


class ErrorClassPlugin(Plugin):
    """
    Base class for ErrorClass plugins. Subclass this class and declare the
    exceptions that you wish to handle as attributes of the subclass.
    """
    __metaclass__ = MetaErrorClass
    score = 1000
    errorClasses = ()

    def addError(self, test, err):
        err_cls, a, b = err
        if not isclass(err_cls):
            return
        classes = [e[0] for e in self.errorClasses]
        if filter(lambda c: issubclass(err_cls, c), classes):
            return True

    def prepareTestResult(self, result):
        if not hasattr(result, 'errorClasses'):
            self.patchResult(result)
        for cls, (storage_attr, label, isfail) in self.errorClasses:
            if cls not in result.errorClasses:
                storage = getattr(result, storage_attr, [])
                setattr(result, storage_attr, storage)
                result.errorClasses[cls] = (storage, label, isfail)

    def patchResult(self, result):
        result.printLabel = print_label_patch(result)
        result._orig_addError, result.addError = \
            result.addError, add_error_patch(result)
        result._orig_wasSuccessful, result.wasSuccessful = \
            result.wasSuccessful, wassuccessful_patch(result)
        if hasattr(result, 'printErrors'):
            result._orig_printErrors, result.printErrors = \
                result.printErrors, print_errors_patch(result)
        if hasattr(result, 'addSkip'):
            result._orig_addSkip, result.addSkip = \
                result.addSkip, add_skip_patch(result)
        result.errorClasses = {}


def add_error_patch(result):
    """Create a new addError method to patch into a result instance
    that recognizes the errorClasses attribute and deals with
    errorclasses correctly.
    """
    return make_instancemethod(NoseTextTestResult.addError, result)


def print_errors_patch(result):
    """Create a new printErrors method that prints errorClasses items
    as well.
    """
    return make_instancemethod(NoseTextTestResult.printErrors, result)


def print_label_patch(result):
    """Create a new printLabel method that prints errorClasses items
    as well.
    """
    return make_instancemethod(NoseTextTestResult.printLabel, result)


def wassuccessful_patch(result):
    """Create a new wasSuccessful method that checks errorClasses for
    exceptions that were put into other slots than error or failure
    but that still count as not success.
    """
    return make_instancemethod(NoseTextTestResult.wasSuccessful, result)


def add_skip_patch(result):
    """Create a new addSkip method to patch into a result instance
    that delegates to addError.
    """
    return make_instancemethod(NoseTextTestResult.addSkip, result)


if __name__ == '__main__':
    import doctest

    doctest.testmod()



#!/usr/bin/env python

"""Test the copy of nose in this directory, by running that nose against itself.

You can test nose using nose in other ways, but if you don't use this script,
you might have one installation of nose testing another installation, which is
not supported.
"""

# More detail:

# In the absence of some sort of deep renaming magic, nose can't reasonably
# test a different installation of itself, given the existence of the global
# module registry sys.modules .

# If installed system-wide with setuptools, setuptools (via the site-packages
# easy-install.pth) takes you at your word and ensures that the installed nose
# comes first on sys.path .  So the only way to test a copy of nose other than
# the installed one is to install that version (e.g. by running python setup.py
# develop).

# This script provides a way of running nose on nose's own tests without
# installing the version to be tested, nor uninstalling the currently-installed
# version.

import glob
import os
import sys

import importlib.metadata


# replacement for old pkg_resources
def importlib_env(search_paths):
    distributions = {}
    for path in search_paths:
        try:
            for distribution in importlib.metadata.distributions(path=path):
                distributions[distribution.metadata['Name']] = distribution
        except Exception as e:
            print(f"Error processing path {path}: {e}")
    return distributions


if __name__ == "__main__":
    this_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
    lib_dirs = [this_dir]
    test_dir = this_dir

    lib_dirs = glob.glob(os.path.join(this_dir, 'build', 'lib*'))
    test_dir = os.path.join(this_dir, 'build', 'tests')
    if not os.path.isdir(test_dir):
        raise Exception(
            "Error: %s does not exist.  Use the setup.py 'build_tests' command to create it." % (
                test_dir,))

    env = importlib_env(lib_dirs)

    distributions = env["nose-py3"]
    if not distributions:
        raise Exception("No nose distibution has been built")
    elif len(distributions) != 1:
        raise Exception("More than one nose distribution found")

    dist = distributions[0]
    dist.activate()

    # Always make sure our chosen test dir is first on the path
    sys.path.insert(0, test_dir)
    import nose

    nose.run_exit()



import os
import sys
import unittest
from difflib import ndiff

from io import StringIO

from nose import loader
from nose import suite
from nose.config import Config
from nose.plugins.allmodules import AllModules
from nose.plugins.manager import PluginManager
from nose.plugins.skip import Skip
from nose.result import TextTestResult

try:
    # 2.7+
    from unittest.runner import _WritelnDecorator
except ImportError:
    from unittest import _WritelnDecorator

support = os.path.abspath(os.path.join(os.path.dirname(__file__), 'support'))


class TestNoseTestLoader(unittest.TestCase):

    def setUp(self):
        self._mods = sys.modules.copy()
        suite.ContextSuiteFactory.suiteClass = TreePrintContextSuite

    def tearDown(self):
        to_del = [m for m in sys.modules.keys() if
                  m not in self._mods]
        if to_del:
            for mod in to_del:
                del sys.modules[mod]
        sys.modules.update(self._mods)
        suite.ContextSuiteFactory.suiteClass = suite.ContextSuite

    def test_load_from_name_file(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package1')
        l = loader.TestLoader(workingDir=wd)

        file_suite = l.loadTestsFromName('tests/test_example_function.py')
        file_suite(res)
        assert not res.errors, res.errors
        assert not res.failures, res.failures

    def test_load_from_name_dot(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package1')
        l = loader.TestLoader(workingDir=wd)
        dir_suite = l.loadTestsFromName('.')
        dir_suite(res)
        assert not res.errors, res.errors
        assert not res.failures, res.failures

    def test_load_from_name_file_callable(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package1')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromName(
            'tests/test_example_function.py:test_times_two')
        suite(res)
        assert not res.errors, res.errors
        assert not res.failures, res.failures
        self.assertEqual(res.testsRun, 1)

    def test_fixture_context(self):
        config = Config(ignoreFiles=[],
                        plugins=PluginManager(plugins=[AllModules()]))
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(config=config, workingDir=wd)
        dir_suite = l.loadTestsFromName('.')
        dir_suite(res)

        m = sys.modules['test_pak']
        print
        "test pak state", m.state

        assert not res.errors, res.errors
        assert not res.failures, res.failures
        self.assertEqual(res.testsRun, 6)

        # Expected order of calls
        expect = ['test_pak.setup',
                  'test_pak.test_mod.setup',
                  'test_pak.test_mod.test_add',
                  'test_pak.test_mod.test_minus',
                  'test_pak.test_mod.teardown',
                  'test_pak.test_sub.setup',
                  'test_pak.test_sub.test_sub_init',
                  'test_pak.test_sub.test_mod.setup',
                  'test_pak.test_sub.test_mod.TestMaths.setup_class',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_div',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_two_two',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.teardown_class',
                  'test_pak.test_sub.test_mod.test',
                  'test_pak.test_sub.test_mod.teardown',
                  'test_pak.test_sub.teardown',
                  'test_pak.teardown']
        self.assertEqual(len(m.state), len(expect))
        for item in m.state:
            self.assertEqual(item, expect.pop(0))

    def test_fixture_context_name_is_module(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromName('test_pak.test_mod')
        suite(res)

        assert 'test_pak' in sys.modules, \
            "Context did not load test_pak"
        m = sys.modules['test_pak']
        print
        "test pak state", m.state
        expect = ['test_pak.setup',
                  'test_pak.test_mod.setup',
                  'test_pak.test_mod.test_add',
                  'test_pak.test_mod.test_minus',
                  'test_pak.test_mod.teardown',
                  'test_pak.teardown']
        self.assertEqual(len(m.state), len(expect))
        for item in m.state:
            self.assertEqual(item, expect.pop(0))

    def test_fixture_context_name_is_test_function(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromName('test_pak.test_mod:test_add')
        suite(res)

        assert 'test_pak' in sys.modules, \
            "Context did not load test_pak"
        m = sys.modules['test_pak']
        print
        "test pak state", m.state
        expect = ['test_pak.setup',
                  'test_pak.test_mod.setup',
                  'test_pak.test_mod.test_add',
                  'test_pak.test_mod.teardown',
                  'test_pak.teardown']
        self.assertEqual(len(m.state), len(expect))
        for item in m.state:
            self.assertEqual(item, expect.pop(0))

    def test_fixture_context_name_is_test_class(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromName(
            'test_pak.test_sub.test_mod:TestMaths')
        suite(res)

        assert 'test_pak' in sys.modules, \
            "Context did not load test_pak"
        m = sys.modules['test_pak']
        # print "test pak state", m.state
        expect = ['test_pak.setup',
                  'test_pak.test_sub.setup',
                  'test_pak.test_sub.test_mod.setup',
                  'test_pak.test_sub.test_mod.TestMaths.setup_class',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_div',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_two_two',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.teardown_class',
                  'test_pak.test_sub.test_mod.teardown',
                  'test_pak.test_sub.teardown',
                  'test_pak.teardown']
        self.assertEqual(m.state, expect, diff(expect, m.state))

    def test_fixture_context_name_is_test_class_test(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromName(
            'test_pak.test_sub.test_mod:TestMaths.test_div')
        suite(res)

        assert 'test_pak' in sys.modules, \
            "Context not load test_pak"
        m = sys.modules['test_pak']
        print
        "test pak state", m.state
        expect = ['test_pak.setup',
                  'test_pak.test_sub.setup',
                  'test_pak.test_sub.test_mod.setup',
                  'test_pak.test_sub.test_mod.TestMaths.setup_class',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_div',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.teardown_class',
                  'test_pak.test_sub.test_mod.teardown',
                  'test_pak.test_sub.teardown',
                  'test_pak.teardown']
        self.assertEqual(m.state, expect, diff(expect, m.state))

    def test_fixture_context_multiple_names(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromNames(
            ['test_pak.test_sub.test_mod:TestMaths.test_div',
             'test_pak.test_sub.test_mod:TestMaths.test_two_two',
             'test_pak.test_mod:test_add'])
        print
        suite
        suite(res)
        assert not res.errors, res.errors
        assert not res.failures, res.failures
        assert 'test_pak' in sys.modules, \
            "Context not load test_pak"
        m = sys.modules['test_pak']
        print
        "test pak state", m.state
        expect = ['test_pak.setup',
                  'test_pak.test_sub.setup',
                  'test_pak.test_sub.test_mod.setup',
                  'test_pak.test_sub.test_mod.TestMaths.setup_class',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_div',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_two_two',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.teardown_class',
                  'test_pak.test_sub.test_mod.teardown',
                  'test_pak.test_sub.teardown',
                  'test_pak.test_mod.setup',
                  'test_pak.test_mod.test_add',
                  'test_pak.test_mod.teardown',
                  'test_pak.teardown']
        self.assertEqual(m.state, expect, diff(expect, m.state))

    def test_fixture_context_multiple_names_some_common_ancestors(self):
        stream = _WritelnDecorator(StringIO())
        res = TextTestResult(stream, 0, 2)
        wd = os.path.join(support, 'ltfn')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromNames(
            ['test_pak1.test_mod',
             'test_pak2:test_two_two',
             'test_pak1:test_one_one'])
        print
        suite
        suite(res)
        res.printErrors()
        print
        stream.getvalue()
        assert not res.errors, res.errors
        assert not res.failures, res.failures
        assert 'state' in sys.modules, \
            "Context not load state module"
        m = sys.modules['state']
        print
        "state", m.called

        expect = ['test_pak1.setup',
                  'test_pak1.test_mod.setup',
                  'test_pak1.test_mod.test_one_mod_one',
                  'test_pak1.test_mod.teardown',
                  'test_pak1.test_one_one',
                  'test_pak1.teardown',
                  'test_pak2.setup',
                  'test_pak2.test_two_two',
                  'test_pak2.teardown']
        self.assertEqual(m.called, expect, diff(expect, m.called))

    def test_fixture_context_multiple_names_no_common_ancestors(self):
        stream = _WritelnDecorator(StringIO())
        res = TextTestResult(stream, 0, 2)
        wd = os.path.join(support, 'ltfn')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromNames(
            ['test_pak1.test_mod',
             'test_pak2:test_two_two',
             'test_mod'])
        print
        suite
        suite(res)
        res.printErrors()
        print
        stream.getvalue()
        assert not res.errors, res.errors
        assert not res.failures, res.failures
        assert 'state' in sys.modules, \
            "Context not load state module"
        m = sys.modules['state']
        print
        "state", m.called

        expect = ['test_pak1.setup',
                  'test_pak1.test_mod.setup',
                  'test_pak1.test_mod.test_one_mod_one',
                  'test_pak1.test_mod.teardown',
                  'test_pak1.teardown',
                  'test_pak2.setup',
                  'test_pak2.test_two_two',
                  'test_pak2.teardown',
                  'test_mod.setup',
                  'test_mod.test_mod',
                  'test_mod.teardown']
        self.assertEqual(m.called, expect, diff(expect, m.called))

    def test_mod_setup_fails_no_tests_run(self):
        ctx = os.path.join(support, 'ctx')
        l = loader.TestLoader(workingDir=ctx)
        suite = l.loadTestsFromName('mod_setup_fails.py')

        res = unittest.TestResult()
        suite(res)

        assert res.errors
        assert not res.failures, res.failures
        assert res.testsRun == 0, \
            "Expected to run 0 tests but ran %s" % res.testsRun

    def test_mod_setup_skip_no_tests_run_no_errors(self):
        config = Config(plugins=PluginManager(plugins=[Skip()]))
        ctx = os.path.join(support, 'ctx')
        l = loader.TestLoader(workingDir=ctx, config=config)
        suite = l.loadTestsFromName('mod_setup_skip.py')

        res = unittest.TestResult()
        suite(res)

        assert not suite.was_setup, "Suite setup did not fail"
        assert not res.errors, res.errors
        assert not res.failures, res.failures
        assert res.skipped
        assert res.testsRun == 0, \
            "Expected to run 0 tests but ran %s" % res.testsRun

    def test_mod_import_skip_one_test_no_errors(self):
        config = Config(plugins=PluginManager(plugins=[Skip()]))
        ctx = os.path.join(support, 'ctx')
        l = loader.TestLoader(workingDir=ctx, config=config)
        suite = l.loadTestsFromName('mod_import_skip.py')

        res = unittest.TestResult()
        suite(res)

        assert not res.errors, res.errors
        assert not res.failures, res.failures
        assert res.testsRun == 1, \
            "Expected to run 1 tests but ran %s" % res.testsRun

    def test_failed_import(self):
        ctx = os.path.join(support, 'ctx')
        l = loader.TestLoader(workingDir=ctx)
        suite = l.loadTestsFromName('no_such_module.py')

        res = TextTestResult(
            stream=_WritelnDecorator(sys.stdout),
            descriptions=0, verbosity=1)
        suite(res)

        print
        res.errors
        res.printErrors()
        assert res.errors, "Expected errors but got none"
        assert not res.failures, res.failures
        assert res.testsRun == 1, \
            "Expected to run 1 tests but ran %s" % res.testsRun

    def test_failed_import_module_name(self):
        ctx = os.path.join(support, 'ctx')
        l = loader.TestLoader(workingDir=ctx)
        suite = l.loadTestsFromName('no_such_module')

        res = TextTestResult(
            stream=_WritelnDecorator(sys.stdout),
            descriptions=0, verbosity=1)
        suite(res)
        print
        res.errors
        res.printErrors()
        assert res.errors, "Expected errors but got none"
        assert not res.failures, res.failures
        err = res.errors[0][0].test.exc_class
        assert err is ImportError, \
            "Expected import error, got %s" % err

    def test_load_nonsense_name(self):
        ctx = os.path.join(support, 'ctx')
        l = loader.TestLoader(workingDir=ctx)
        suite = l.loadTestsFromName('fred!')

        res = TextTestResult(
            stream=_WritelnDecorator(sys.stdout),
            descriptions=0, verbosity=1)
        suite(res)
        print
        res.errors
        assert res.errors, "Expected errors but got none"
        assert not res.failures, res.failures

    def test_generator_with_closure(self):
        """Test that a generator test can employ a closure

        Issue #3. If the generator binds early, the last value
        of the closure will be seen for each generated test and
        the tests will fail.
        """
        gen = os.path.join(support, 'gen')
        l = loader.TestLoader(workingDir=gen)
        suite = l.loadTestsFromName('test')
        res = TextTestResult(
            stream=_WritelnDecorator(sys.stdout),
            descriptions=0, verbosity=1)
        suite(res)
        assert not res.errors
        self.assertEqual(res.testsRun, 5)

    def test_issue_269(self):
        """Test classes that raise exceptions in __init__ do not stop test run
        """
        wdir = os.path.join(support, 'issue269')
        l = loader.TestLoader(workingDir=wdir)
        suite = l.loadTestsFromName('test_bad_class')
        res = TextTestResult(
            stream=_WritelnDecorator(sys.stdout),
            descriptions=0, verbosity=1)
        suite(res)
        print
        res.errors
        self.assertEqual(len(res.errors), 1)
        assert 'raise Exception("pow")' in res.errors[0][1]

    def test_load_from_file(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromName('test_pak/test_sub/__init__.py')
        suite(res)

        assert 'test_pak' in sys.modules, \
            "Context did not load test_pak"
        m = sys.modules['test_pak']
        print
        "test pak state", m.state
        expect = ['test_pak.setup',
                  'test_pak.test_sub.setup',
                  'test_pak.test_sub.test_sub_init',
                  'test_pak.test_sub.teardown',
                  'test_pak.teardown']
        self.assertEqual(len(m.state), len(expect))
        for item in m.state:
            self.assertEqual(item, expect.pop(0))

    def test_load_from_sub_package(self):
        res = unittest.TestResult()
        wd = os.path.join(support, 'package2')
        l = loader.TestLoader(workingDir=wd)
        suite = l.loadTestsFromName('test_pak.test_sub')
        suite(res)

        assert 'test_pak' in sys.modules, \
            "Context did not load test_pak"
        m = sys.modules['test_pak']
        print
        "test pak state", m.state
        expect = ['test_pak.setup',
                  'test_pak.test_sub.setup',
                  'test_pak.test_sub.test_sub_init',
                  'test_pak.test_sub.test_mod.setup',
                  'test_pak.test_sub.test_mod.TestMaths.setup_class',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_div',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.setup',
                  'test_pak.test_sub.test_mod.TestMaths.test_two_two',
                  'test_pak.test_sub.test_mod.TestMaths.teardown',
                  'test_pak.test_sub.test_mod.TestMaths.teardown_class',
                  'test_pak.test_sub.test_mod.test',
                  'test_pak.test_sub.test_mod.teardown',
                  'test_pak.test_sub.teardown',
                  'test_pak.teardown']
        self.assertEqual(len(m.state), len(expect))
        for item in m.state:
            self.assertEqual(item, expect.pop(0))


# used for comparing lists
def diff(a, b):
    return '\n' + '\n'.join([l for l in ndiff(a, b)
                             if not l.startswith('  ')])


# used for context debugging
class TreePrintContextSuite(suite.ContextSuite):
    indent = ''

    def setUp(self):
        print
        self, 'setup -->'
        suite.ContextSuite.setUp(self)
        TreePrintContextSuite.indent += '  '

    def tearDown(self):
        TreePrintContextSuite.indent = TreePrintContextSuite.indent[:-2]
        try:
            suite.ContextSuite.tearDown(self)
        finally:
            print
            self, 'teardown <--'

    def __repr__(self):

        return '%s<%s>' % (self.indent,
                           getattr(self.context, '__name__', self.context))

    __str__ = __repr__


if __name__ == '__main__':
    # import logging
    # logging.basicConfig() #level=logging.DEBUG)
    # logging.getLogger('nose.suite').setLevel(logging.DEBUG)
    unittest.main()



class TestFunctionalTest(object):
    counter = 0

    @classmethod
    def setup_class(cls):
        cls.counter += 1

    @classmethod
    def teardown_class(cls):
        cls.counter -= 1

    def _run(self):
        assert self.counter == 1

    def test1(self):
        self._run()

    def test2(self):
        self._run()



import os
import re
import sys
import tempfile
import unittest

from pickle import load

from nose.plugins import PluginTester
from nose.plugins.builtin import Doctest
from nose.plugins.builtin import TestId

support = os.path.join(os.path.dirname(__file__), 'support')
idfile = tempfile.mktemp()
test_part = re.compile(r'(#\d+)? +([^(]+)')


def teardown():
    try:
        os.remove(idfile)
    except OSError:
        pass


class TestDiscoveryMode(PluginTester, unittest.TestCase):
    activate = '--with-id'
    plugins = [TestId()]
    args = ['-v', '--id-file=%s' % idfile]
    suitepath = os.path.join(support, 'idp')

    def test_ids_added_to_output(self):
        # print '>' * 70
        # print str(self.output)
        # print '<' * 70

        for line in self.output:
            if line.startswith('='):
                break
            if not line.strip():
                continue
            if 'test_gen' in line and not '(0,)' in line:
                assert not line.startswith('#'), \
                    "Generated test line '%s' should not have id" % line
            else:
                assert line.startswith('#'), \
                    "Test line '%s' missing id" % line.strip()

    # test that id file is written
    def test_id_file_contains_ids_seen(self):
        assert os.path.exists(idfile)
        fh = open(idfile, 'rb')
        ids = load(fh)['ids']
        fh.close()
        assert ids
        assert ids.keys()
        self.assertEqual(map(int, ids.keys()), ids.keys())
        assert ids.values()


class TestLoadNamesMode(PluginTester, unittest.TestCase):
    """NOTE that this test passing requires the previous test case to
    be run! (Otherwise the ids file will not exist)
    """
    activate = '--with-id'
    plugins = [TestId()]
    # Not a typo: # is optional before ids
    args = ['-v', '--id-file=%s' % idfile, '2', '#5']
    suitepath = None

    def makeSuite(self):
        return None

    def test_load_ids(self):
        # print '#' * 70
        # print str(self.output)
        # print '#' * 70

        for line in self.output:
            if line.startswith('#'):
                assert line.startswith('#2 ') or line.startswith('#5 '), \
                    "Unexpected test line '%s'" % line
        assert os.path.exists(idfile)
        fh = open(idfile, 'rb')
        ids = load(fh)
        fh.close()
        assert ids
        assert ids.keys()
        ids = ids['ids']
        self.assertEqual(filter(lambda i: int(i), ids.keys()), ids.keys())
        assert len(ids.keys()) > 2


class TestLoadNamesMode_2(PluginTester, unittest.TestCase):
    """NOTE that this test passing requires the previous test case to
    be run! (Otherwise the ids file will not exist)

    Tests that generators still only have id on one line
    """
    activate = '--with-id'
    plugins = [TestId()]
    args = ['-v', '--id-file=%s' % idfile, '9']
    suitepath = None

    def makeSuite(self):
        return None

    def test_load_ids(self):
        # print '%' * 70
        # print str(self.output)
        # print '%' * 70

        count = 0
        for line in self.output:
            if line.startswith('#'):
                count += 1
        self.assertEqual(count, 1)
        teardown()


class TestWithDoctest_1(PluginTester, unittest.TestCase):
    activate = '--with-id'
    plugins = [Doctest(), TestId()]
    args = ['-v', '--id-file=%s' % idfile, '--with-doctest']
    suitepath = os.path.join(support, 'idp')

    def test_doctests_get_ids(self):
        # print '>' * 70
        # print str(self.output)
        # print '>' * 70

        last = None
        for line in self.output:
            if line.startswith('='):
                break
            if not line.strip():
                continue
            # assert line startswith # or test part matches last
            m = test_part.match(line.rstrip())
            assert m
            idx, name = m.groups()
            assert idx or last is None or name == last, \
                "Expected an id on line %s" % line.strip()
            last = name

        fh = open(idfile, 'rb')
        ids = load(fh)['ids']
        fh.close()
        for key, (file, mod, call) in ids.items():
            assert mod != 'doctest', \
                "Doctest test was incorrectly identified as being part of " \
                "the doctest module itself (#%s)" % key


class TestWithDoctest_2(PluginTester, unittest.TestCase):
    activate = '--with-id'
    plugins = [Doctest(), TestId()]
    args = ['-v', '--id-file=%s' % idfile, '--with-doctest', '#2']
    suitepath = None

    def setUp(self):
        sys.path.insert(0, os.path.join(support, 'idp'))
        super(TestWithDoctest_2, self).setUp()

    def tearDown(self):
        sys.path.remove(os.path.join(support, 'idp'))
        super(TestWithDoctest_2, self).tearDown()

    def makeSuite(self):
        return None

    def test_load_ids_doctest(self):
        print
        '*' * 70
        print
        str(self.output)
        print
        '*' * 70

        assert 'Doctest: exm.add_one ... FAIL' in self.output

        count = 0
        for line in self.output:
            if line.startswith('#'):
                count += 1
        self.assertEqual(count, 1)
        teardown()


class TestWithDoctestFileTests_1(PluginTester, unittest.TestCase):
    activate = '--with-id'
    plugins = [Doctest(), TestId()]
    args = ['-v', '--id-file=%s' % idfile, '--with-doctest',
            '--doctest-extension=.txt']
    suitepath = os.path.join(support, 'dtt', 'docs')

    def test_docfile_tests_get_ids(self):
        print
        '>' * 70
        print
        str(self.output)
        print
        '>' * 70

        last = None
        for line in self.output:
            if line.startswith('='):
                break
            # assert line startswith # or test part matches last
            if not line.strip():
                continue
            m = test_part.match(line.rstrip())
            assert m, "line %s does not match expected pattern" % line.strip()
            idx, name = m.groups()
            assert idx or last is None or name == last, \
                "Expected an id on line %s" % line.strip()

            last = name
        fh = open(idfile, 'rb')
        ids = load(fh)['ids']
        fh.close()
        for key, (file, mod, call) in ids.items():
            assert mod != 'doctest', \
                "Doctest test was incorrectly identified as being part of " \
                "the doctest module itself (#%s)" % key


class TestWithDoctestFileTests_2(PluginTester, unittest.TestCase):
    activate = '--with-id'
    plugins = [Doctest(), TestId()]
    args = ['-v', '--id-file=%s' % idfile, '--with-doctest',
            '--doctest-extension=.txt', '2']
    suitepath = None

    def setUp(self):
        sys.path.insert(0, os.path.join(support, 'dtt', 'docs'))
        super(TestWithDoctestFileTests_2, self).setUp()

    def tearDown(self):
        sys.path.remove(os.path.join(support, 'dtt', 'docs'))
        super(TestWithDoctestFileTests_2, self).tearDown()

    def makeSuite(self):
        return None

    def test_load_from_name_id_docfile_test(self):
        print
        '*' * 70
        print
        str(self.output)
        print
        '*' * 70

        assert 'Doctest: errdoc.txt ... FAIL' in self.output

        count = 0
        for line in self.output:
            if line.startswith('#'):
                count += 1
        assert count == 1
        teardown()


if __name__ == '__main__':
    import logging

    logging.basicConfig()
    l = logging.getLogger('nose.plugins.testid')
    l.setLevel(logging.DEBUG)

    try:
        unittest.main()
    finally:
        teardown()



from nose.tools import eq_

called = []


def outer_setup():
    called.append('outer_setup')


def outer_teardown():
    called.append('outer_teardown')


def inner_setup():
    called.append('inner_setup')


def inner_teardown():
    called.append('inner_teardown')


def test_gen():
    called[:] = []
    for i in range(0, 5):
        yield check, i


def check(i):
    expect = ['outer_setup']
    for x in range(0, i):
        expect.append('inner_setup')
        expect.append('inner_teardown')
    expect.append('inner_setup')
    eq_(called, expect)


test_gen.setup = outer_setup
test_gen.teardown = outer_teardown
check.setup = inner_setup
check.teardown = inner_teardown


class TestClass(object):
    def setup(self):
        print("")
        "setup called in", self
        self.called = ['setup']

    def teardown(self):
        print("")
        "teardown called in", self
        eq_(self.called, ['setup'])
        self.called.append('teardown')

    def test(self):
        print("")
        "test called in", self
        for i in range(0, 5):
            yield self.check, i

    def check(self, i):
        print("")
        "check called in", self
        expect = ['setup']
        # for x in range(0, i):
        #    expect.append('setup')
        #    expect.append('teardown')
        # expect.append('setup')
        eq_(self.called, expect)



import os
import unittest
from io import StringIO
from nose.core import TestProgram
from nose.config import Config
from nose.result import TextTestResult

here = os.path.dirname(__file__)
support = os.path.join(here, 'support')


class TestRunner(unittest.TextTestRunner):
    def _makeResult(self):
        self.result = TextTestResult(
            self.stream, self.descriptions, self.verbosity)
        return self.result


class TestBuggyGenerators(unittest.TestCase):
    def test_run_buggy_generators(self):
        stream = StringIO()
        runner = TestRunner(stream=stream)
        prog = TestProgram(
            argv=['nosetests',
                  os.path.join(support, 'test_buggy_generators.py')],
            testRunner=runner,
            config=Config(),
            exit=False)
        res = runner.result
        print("")
        stream.getvalue()
        self.assertEqual(res.testsRun, 9,
                         "Expected to run 9 tests, ran %s" % res.testsRun)
        assert not res.wasSuccessful()
        assert not res.failures



import os
import unittest
import warnings

from io import StringIO

from nose.result import TextTestResult

here = os.path.dirname(__file__)
support = os.path.join(here, 'support')


class TestRunner(unittest.TextTestRunner):
    def _makeResult(self):
        self.result = TextTestResult(
            self.stream, self.descriptions, self.verbosity)
        return self.result


class TestNoseTestCollector(unittest.TestCase):

    def test_skip_works_with_collector(self):
        verbosity = 2
        stream = StringIO()
        runner = TestRunner(stream=stream, verbosity=verbosity)
        pwd = os.getcwd()

        # we don't need to see our own warnings
        warnings.filterwarnings(action='ignore',
                                category=RuntimeWarning,
                                module='nose.plugins.manager')

        try:
            os.chdir(os.path.join(support, 'issue038'))
            unittest.TestProgram(
                None, None,
                argv=['test_collector', '-v', 'nose.collector'],
                testRunner=runner)
        except SystemExit:
            pass
        os.chdir(pwd)
        out = stream.getvalue()
        assert runner.result.wasSuccessful()
        assert 'SKIP' in out, "SKIP not found in %s" % out


if __name__ == '__main__':
    unittest.main()



import os

from time import sleep

if 'NOSE_MP_LOG' not in os.environ:
    raise Exception('Environment variable NOSE_MP_LOG is not set')

logfile = os.environ['NOSE_MP_LOG']
killfile = os.environ['NOSE_MP_KILL']


def log(w):
    f = open(logfile, 'a')
    f.write(w + "\n")
    f.close()


def touch_killfile():
    f = open(killfile, 'wb')
    f.close()


# make sure all tests in this file are dispatched to the same subprocess
def setup():
    '''global logfile
    logfile = mktemp()
    print "tempfile is:",logfile'''
    log('setup')


def test_timeout():
    log('test_timeout')
    touch_killfile()
    sleep(2)
    log('test_timeout_finished')


# check timeout will not prevent remaining tests dispatched to the same subprocess to continue to run
def test_pass():
    log('test_pass')


def teardown():
    log('teardown')
    touch_killfile()
    sleep(10)
    log('teardown_finished')



import sys
import textwrap
import tokenize
import unittest

from io import StringIO

from nose.inspector import inspect_traceback, Expander, tbsource


class TestExpander(unittest.TestCase):

    def test_simple_inspect_frame(self):
        src = StringIO('a > 2')
        lc = {'a': 2}
        gb = {}
        exp = Expander(lc, gb)

        for tok in tokenize.generate_tokens(src.readline):
            exp(*tok)
        # print("'%s'" % exp.expanded_source)
        self.assertEqual(exp.expanded_source.strip(), '2 > 2')

    def test_inspect_traceback_continued(self):
        a = 6
        out = ''
        try:
            assert a < 1, \
                "This is a multline expression"
        except AssertionError:
            et, ev, tb = sys.exc_info()
            out = inspect_traceback(tb)
            # print("'%s'" % out.strip())
            self.assertEqual(out.strip(),
                             '>>  assert 6 < 1, \\\n        '
                             '"This is a multline expression"')

    def test_get_tb_source_simple(self):
        # no func frame
        try:
            assert False
        except AssertionError:
            et, ev, tb = sys.exc_info()
            lines, lineno = tbsource(tb, 1)
            self.assertEqual(''.join(lines).strip(), 'assert False')
            self.assertEqual(lineno, 0)

    def test_get_tb_source_func(self):
        # func frame
        def check_even(n):
            print(n)
            assert n % 2 == 0

        try:
            check_even(1)
        except AssertionError:
            et, ev, tb = sys.exc_info()
            lines, lineno = tbsource(tb)
            out = textwrap.dedent(''.join(lines))
            first_line = '    print(n)\n'
            self.assertEqual(out,
                             first_line +
                             '    assert n % 2 == 0\n'
                             'try:\n'
                             '    check_even(1)\n'
                             'except AssertionError:\n'
                             '    et, ev, tb = sys.exc_info()\n'
                             )
            self.assertEqual(lineno, 3)

        # FIXME 2 func frames

    def test_pick_tb_lines(self):
        try:
            val = "fred"

            def defred(n):
                return n.replace('fred', '')

            assert defred(val) == 'barney', "Fred - fred != barney?"
        except AssertionError:
            et, ev, tb = sys.exc_info()
            out = inspect_traceback(tb)
            # print("'%s'" % out.strip())
            self.assertEqual(out.strip(),
                             ">>  assert defred('fred') == 'barney', "
                             '"Fred - fred != barney?"')
        try:
            val = "fred"

            def defred(n):
                return n.replace('fred', '')

            assert defred(val) == 'barney', \
                "Fred - fred != barney?"

            def refred(n):
                return n + 'fred'
        except AssertionError:
            et, ev, tb = sys.exc_info()
            out = inspect_traceback(tb)
            # print("'%s'" % out.strip())
            self.assertEqual(out.strip(),
                             ">>  assert defred('fred') == 'barney', "
                             '\\\n        "Fred - fred != barney?"')

        S = {'setup': 1}

        def check_even(n, nn):
            assert S['setup']
            print(n, nn)
            assert n % 2 == 0 or nn % 2 == 0

        try:
            check_even(1, 3)
        except AssertionError:
            et, ev, tb = sys.exc_info()
            out = inspect_traceback(tb)
            print("'%s'" % out.strip())
            print_line = "    print(1, 3)\n"
            self.assertEqual(out.strip(),
                             "assert {'setup': 1}['setup']\n" +
                             print_line +
                             ">>  assert 1 % 2 == 0 or 3 % 2 == 0")

    def test_bug_95(self):
        """Test that inspector can handle multi-line docstrings"""
        try:
            """docstring line 1
            docstring line 2
            """
            a = 2
            assert a == 4
        except AssertionError:
            et, ev, tb = sys.exc_info()
            out = inspect_traceback(tb)
            print("'%s'" % out.strip())
            self.assertEqual(out.strip(),
                             "2 = 2\n"
                             ">>  assert 2 == 4")


if __name__ == '__main__':
    # import logging
    # logging.basicConfig()
    # logging.getLogger('').setLevel(10)
    unittest.main()



from state import called


def setup():
    called.append('test_pak1.test_mod.setup')


def teardown():
    called.append('test_pak1.test_mod.teardown')


def test_one_mod_one():
    called.append('test_pak1.test_mod.test_one_mod_one')
    pass



import os
import unittest

from nose.plugins import PluginTester
from nose.plugins.builtin import FailureDetail, Capture

support = os.path.join(os.path.dirname(__file__), 'support')


class TestFailureDetailWorks(PluginTester, unittest.TestCase):
    activate = '-d'
    plugins = [FailureDetail()]
    args = ['-v']
    suitepath = os.path.join(support, 'issue072')

    def test_assert_info_in_output(self):
        print
        print
        '!' * 70
        print
        str(self.output)
        print
        '!' * 70
        print
        assert '>>  assert 4 == 2' in str(self.output)


class TestFailureDetailWorksWhenChained(PluginTester, unittest.TestCase):
    activate = '-d'
    plugins = [FailureDetail(), Capture()]
    args = ['-v']
    suitepath = os.path.join(support, 'issue072')

    def test_assert_info_and_capt_stdout_in_output(self):
        out = str(self.output)
        print
        print
        'x' * 70
        print
        out
        print
        'x' * 70
        print

        assert '>>  assert 4 == 2' in out, \
            "Assert info not found in chained output"
        assert 'something' in out, \
            "Captured stdout not found in chained output"


if __name__ == '__main__':
    unittest.main()



class base:
    @classmethod
    def setup_class(cls):
        cls.inited = 1

    @classmethod
    def teardown_class(cls):
        cls.inited = 0

    def test1(self):
        assert self.inited

    def test2(self):
        assert self.inited


class testa(base):
    pass


class testb(base):
    pass



import os

from test_multiprocessing import MPTestBase


class TestMPNameError(MPTestBase):
    processes = 2
    suitepath = os.path.join(os.path.dirname(__file__), 'support', 'nameerror.py')

    def runTest(self):
        print
        str(self.output)
        assert 'NameError' in self.output
        assert "'undefined_variable' is not defined" in self.output



import logging
import unittest
from traceback import format_tb

from nose.pyversion import is_base_exception

log = logging.getLogger(__name__)

__all__ = ['Failure']


class Failure(unittest.TestCase):
    """Unloadable or unexecutable test.

    A Failure case is placed in a test suite to indicate the presence of a
    test that could not be loaded or executed. A common example is a test
    module that fails to import.

    """
    __test__ = False  # do not collect

    def __init__(self, exc_class, exc_val, tb=None, address=None):
        log.debug("A failure! %s %s %s", exc_class, exc_val, format_tb(tb))
        self.exc_class = exc_class
        self.exc_val = exc_val
        self.tb = tb
        self._address = address
        unittest.TestCase.__init__(self)

    def __str__(self):
        return "Failure: %s (%s)" % (
            getattr(self.exc_class, '__name__', self.exc_class), self.exc_val)

    def address(self):
        return self._address

    def runTest(self):
        if self.tb is not None:
            if is_base_exception(self.exc_val):
                raise self.exc_val.with_traceback(self.tb)
            raise self.exc_class(self.exc_val).with_traceback(self.tb)
        else:
            raise self.exc_class(self.exc_val)



from nose.plugins.skip import SkipTest
from nose.tools import *
from nose.twistedtools import *

try:
    from twisted.internet.defer import Deferred
    from twisted.internet.error import DNSLookupError
except ImportError:
    raise SkipTest('twisted not available; skipping')

_multiprocess_ = False


def teardown():
    # print "stopping reactor"
    stop_reactor()


class CustomError(Exception):
    pass


# FIXME move all dns-using tests to functional

# Should succeed unless google is down
# @deferred
def test_resolve():
    return reactor.resolve("www.google.com")


test_resolve = deferred()(test_resolve)


# Raises TypeError because the function does not return a Deferred
# @raises(TypeError)
# @deferred()
def test_raises_bad_return():
    print
    reactor
    reactor.resolve("www.python.org")


test_raises_bad_return = raises(TypeError)(deferred()(test_raises_bad_return))


# Check we propagate twisted Failures as Exceptions
# (XXX this test might take some time: find something better?)
# @raises(DNSLookupError)
# @deferred()
def test_raises_twisted_error():
    return reactor.resolve("x.y.invalid")


test_raises_twisted_error = raises(DNSLookupError)(
    deferred()(test_raises_twisted_error))


# Check we detect Exceptions inside the callback chain
# @raises(CustomError)
# @deferred(timeout=1.0)
def test_raises_callback_error():
    d = Deferred()

    def raise_error(_):
        raise CustomError()

    def finish():
        d.callback(None)

    d.addCallback(raise_error)
    reactor.callLater(0.01, finish)
    return d


test_raises_callback_error = raises(CustomError)(
    deferred(timeout=1.0)(test_raises_callback_error))


# Check we detect Exceptions inside the test body
# @raises(CustomError)
# @deferred(timeout=1.0)
def test_raises_plain_error():
    raise CustomError


test_raises_plain_error = raises(CustomError)(
    deferred(timeout=1.0)(test_raises_plain_error))


# The deferred is triggered before the timeout: ok
# @deferred(timeout=1.0)
def test_timeout_ok():
    d = Deferred()

    def finish():
        d.callback(None)

    reactor.callLater(0.01, finish)
    return d


test_timeout_ok = deferred(timeout=1.0)(test_timeout_ok)


# The deferred is triggered after the timeout: failure
# @raises(TimeExpired)
# @deferred(timeout=0.1)
def test_timeout_expired():
    d = Deferred()

    def finish():
        d.callback(None)

    reactor.callLater(1.0, finish)
    return d


test_timeout_expired = raises(TimeExpired)(
    deferred(timeout=0.1)(test_timeout_expired))

if __name__ == '__main__':
    from nose import runmodule

    runmodule()



import logging
import os
import unittest

from nose.config import Config

support = os.path.join(os.path.dirname(__file__), 'support')


class TestConfigurationFromFile(unittest.TestCase):
    def setUp(self):
        self.cfg_file = os.path.join(support, 'test.cfg')

        # install mock root logger so that these tests don't stomp on
        # the real logging config of the test runner
        class MockLogger(logging.Logger):
            root = logging.RootLogger(logging.WARNING)
            manager = logging.Manager(root)

        self.real_logger = logging.Logger
        self.real_root = logging.root
        logging.Logger = MockLogger
        logging.root = MockLogger.root

    def tearDown(self):
        # reset real root logger
        logging.Logger = self.real_logger
        logging.root = self.real_root

    def test_load_config_file(self):
        c = Config(files=self.cfg_file)
        c.configure(['test_load_config_file'])
        self.assertEqual(c.verbosity, 10)

    def test_config_file_set_by_arg(self):
        c = Config()
        c.configure(['test_config_file_set_by_arg',
                     '-c', self.cfg_file, '-v'])
        # 10 from file, 1 more from cmd line
        self.assertEqual(c.verbosity, 11)


if __name__ == '__main__':
    unittest.main()



import importlib
import importlib.util
import sys, os


# replacement with new importlib function wrapper
def load_source(name, filepath):
    spec = importlib.util.spec_from_file_location(name, filepath)
    if spec is None:
        raise ImportError(f"Error: Can not load the module {name} from {filepath}")

    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    sys.modules[name] = module
    return module


def __bootstrap__():
    """ Import the code in ``noname_wrapped.not_py`` in file as our own name

    This is a simplified version of the wrapper that setuptools writes for
    dynamic libraries when installing.
    """

    here = os.path.join(os.path.dirname(__file__))
    load_source(__name__, os.path.join(here, 'noname_wrapped.not_py'))


__bootstrap__()



from twisted.trial import unittest


class TestTwisted(unittest.TestCase):

    def test(self):
        pass

    def test_fail(self):
        self.fail("I failed")

    def test_error(self):
        raise TypeError("oops, wrong type")

    def test_skip(self):
        raise unittest.SkipTest('skip me')



"""This test will fail if generators bind too early."""

from nose.tools import eq_


def test1():
    def func(_l, _n):
        eq_(len(_l), _n)

    l = []
    for i in xrange(5):
        yield func, l, i
        l.append(None)



import sys
import unittest
import warnings

from nose.plugins.skip import SkipTest
from nose.plugins.errorclass import ErrorClass, ErrorClassPlugin


class TestErrorClassWithStringException(unittest.TestCase):

    def test_string_exception_not_masked(self):
        if sys.version_info >= (3,):
            raise SkipTest("Python 3.x does not support string exceptions")

        class X(Exception):
            pass

        class EP(ErrorClassPlugin):
            xes = ErrorClass(X, label='XXX', isfailure=True)

        warnings.filterwarnings(action='ignore', category=DeprecationWarning)
        try:

            raise Exception("oh no!")
        except:
            exc = sys.exc_info()

        ep = EP()
        self.assertEqual(ep.addError(None, exc), None)



import sys
import unittest
from optparse import OptionParser

from io import StringIO

from nose.config import Config
from nose.plugins import debug


class StubPdb:
    called = False

    def post_mortem(self, tb):
        self.called = True


class TestPdbPlugin(unittest.TestCase):

    def setUp(self):
        self._pdb = debug.pdb
        self._so = sys.stdout
        debug.pdb = StubPdb()

    def tearDown(self):
        debug.pdb = self._pdb
        sys.stdout = self._so

    def test_plugin_api(self):
        p = debug.Pdb()
        p.addOptions
        p.configure
        p.addError
        p.addFailure

    def test_plugin_calls_pdb(self):
        p = debug.Pdb()

        try:
            raise Exception("oops")
        except:
            err = sys.exc_info()

        p.enabled = True
        p.enabled_for_errors = True
        p.enabled_for_failures = True

        p.addError(None, err)
        assert debug.pdb.called, "Did not call pdb.post_mortem on error"

        debug.pdb.called = False
        p.addFailure(None, err)
        assert debug.pdb.called, "Did not call pdb.post_mortem on failure"

    def test_command_line_options_enable(self):
        parser = OptionParser()

        p = debug.Pdb()
        p.addOptions(parser)
        options, args = parser.parse_args(['test_configuration',
                                           '--pdb',
                                           '--pdb-failures'])
        p.configure(options, Config())
        assert p.enabled
        assert p.enabled_for_errors
        assert p.enabled_for_failures

    def test_disabled_by_default(self):
        p = debug.Pdb()
        assert not p.enabled
        assert not p.enabled_for_failures

        parser = OptionParser()
        p.addOptions(parser)
        options, args = parser.parse_args(['test_configuration'])
        p.configure(options, Config())
        assert not p.enabled
        assert not p.enabled_for_errors
        assert not p.enabled_for_failures

    def test_env_settings_enable(self):
        p = debug.Pdb()
        assert not p.enabled
        assert not p.enabled_for_failures

        env = {'NOSE_PDB': '1',
               'NOSE_PDB_FAILURES': '1'}

        parser = OptionParser()
        p.addOptions(parser, env)
        options, args = parser.parse_args(['test_configuration'])
        p.configure(options, Config())
        assert p.enabled
        assert p.enabled_for_errors
        assert p.enabled_for_failures

    def test_real_stdout_restored_before_call(self):

        class CheckStdout(StubPdb):
            def post_mortem(self, tb):
                assert sys.stdout is sys.__stdout__, \
                    "sys.stdout was not restored to sys.__stdout__ " \
                    "before call"

        debug.pdb = CheckStdout()

        patch = StringIO()
        sys.stdout = patch
        p = debug.Pdb()
        p.enabled = True
        p.enabled_for_errors = True

        try:
            raise Exception("oops")
        except:
            err = sys.exc_info()

        p.addError(None, err)
        assert sys.stdout is patch, "sys.stdout was not reset after call"


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from nose.plugins import PluginTester, Plugin
from nose.tools import eq_


class StubPlugin(Plugin):
    def options(self, parser, env=os.environ):
        super(StubPlugin, self).options(parser, env=env)

    def configure(self, options, conf):
        pass


class SomePluginTestCase(PluginTester):
    activate = None  # set this to --with-yourplugin, etc
    plugins = []  # list of plugin instances

    def makeSuite(self):
        class SomeTest(unittest.TestCase):
            def runTest(self):
                raise ValueError("Now do something, plugin!")

        return unittest.TestSuite([SomeTest()])


class TestPluginTester(unittest.TestCase):
    def _runPluginTest(self, test_case):
        loader = unittest.TestLoader()
        suite = loader.loadTestsFromTestCase(test_case)
        res = unittest.TestResult()
        suite(res)
        return res

    def testPluginTesterExecsPlugin(self):
        called = []

        class MockExecPlugin(StubPlugin):
            def configure(self, options, conf):
                called.append('configure')

        class MockExecTestCase(SomePluginTestCase, unittest.TestCase):
            activate = '--with-mockexec'
            plugins = [MockExecPlugin()]

            def test_something_anything(self):
                # here is where the test case would test
                # that the plugin interacted with stub tests
                pass

        res = self._runPluginTest(MockExecTestCase)
        eq_(res.testsRun, 1)
        eq_(called[0], 'configure')


if __name__ == '__main__':
    unittest.main()



import unittest

from nose.plugins.attrib import attr


def test_one():
    pass


test_one.a = 1
test_one.d = [1, 2]


def test_two():
    pass


test_two.a = 1
test_two.c = 20
test_two.d = [2, 3]


def test_three():
    pass


test_three.b = 1
test_three.d = [1, 3]


class TestClass:
    a = 1

    def test_class_one(self):
        pass

    def test_class_two(self):
        pass

    test_class_two.b = 2

    def test_class_three(self):
        pass


class Something(unittest.TestCase):
    b = 2

    def test_case_one(self):
        pass

    def test_case_two(self):
        pass

    test_case_two.c = 50

    def test_case_three(self):
        pass


class Superclass:
    def test_method(self):
        pass

    test_method.from_super = True


class TestSubclass(Superclass):
    pass


class Static:
    def test_with_static(self):
        pass

    test_with_static.with_static = True

    def static(self):
        pass

    static = staticmethod(static)


class TestClassAndMethodAttrs(unittest.TestCase):
    def test_method(self):
        pass

    test_method.meth_attr = 'method'


TestClassAndMethodAttrs.cls_attr = 'class'


class TestAttrClass:
    from_super = True

    def ends_with_test(self):
        pass

    def test_one(self):
        pass

    def test_two(self):
        pass

    test_two.from_super = False


TestAttrClass = attr('a')(TestAttrClass)


class TestAttrSubClass(TestAttrClass):
    def test_sub_three(self):
        pass


def added_later_test(self):
    pass


TestAttrSubClass.added_later_test = added_later_test



from nose.core import collector, main, run, run_exit, runmodule
from nose.plugins.deprecated import DeprecatedTest
from nose.plugins.skip import SkipTest
from nose.tools import with_setup

__author__ = 'Adam Bilbrough'
__versioninfo__ = (1, 7, 0)
__version__ = '.'.join(map(str, __versioninfo__))

__all__ = [
    'main', 'run', 'run_exit', 'runmodule', 'with_setup',
    'SkipTest', 'DeprecatedTest', 'collector'
    ]



import unittest

import mock

from nose.config import Config
from nose.plugins.builtin import TestId


class TestTestIdPlugin(unittest.TestCase):

    def test_default_id_file_is_in_working_dir(self):
        tid = TestId()
        c = Config()
        opt = mock.Bucket()
        opt.testIdFile = '.noseids'
        tid.configure(opt, c)
        print(tid.idfile)
        assert tid.idfile.startswith(c.workingDir), \
            "%s is not under %s" % (tid.idfile, c.workingDir)


if __name__ == '__main__':
    unittest.main()



"""
Test Result
-----------

Provides a TextTestResult that extends unittest's TextTestResult to
provide support for error classes (such as the builtin skip and
deprecated classes), and hooks for plugins to take over or extend
reporting.
"""

import logging

try:
    from unittest.runner import TextTestResult
except ImportError:
    from unittest import TextTestResult
from nose.config import Config
from nose.util import isclass, ln as _ln  # backwards compat

log = logging.getLogger('nose.result')


def _exception_detail(exc):
    # this is what stdlib module traceback does
    try:
        return str(exc)
    except:
        return '<unprintable %s object>' % type(exc).__name__


class NoseTextTestResult(TextTestResult):
    """Text test result that extends unittest's default test result
    support for a configurable set of errorClasses (eg, Skip,
    Deprecated, TODO) that extend the errors/failures/success triad.
    """

    def __init__(self, stream, descriptions, verbosity, config=None,
                 errorClasses=None):
        if errorClasses is None:
            errorClasses = {}
        self.errorClasses = errorClasses
        if config is None:
            config = Config()
        self.config = config
        TextTestResult.__init__(self, stream, descriptions, verbosity)

    def addSkip(self, test, reason):
        from nose.plugins.skip import SkipTest
        if SkipTest in self.errorClasses:
            storage, label, isfail = self.errorClasses[SkipTest]
            storage.append((test, reason))
            self.printLabel(label, (SkipTest, reason, None))

    def addError(self, test, err):
        """Overrides normal addError to add support for
        errorClasses. If the exception is a registered class, the
        error will be added to the list for that class, not errors.
        """
        ec, ev, tb = err
        try:
            exc_info = self._exc_info_to_string(err, test)
        except TypeError:
            exc_info = self._exc_info_to_string(err)
        for cls, (storage, label, isfail) in self.errorClasses.items():
            if isclass(ec) and issubclass(ec, cls):
                if isfail:
                    test.passed = False
                storage.append((test, exc_info))
                self.printLabel(label, err)
                return
        self.errors.append((test, exc_info))
        test.passed = False
        self.printLabel('ERROR')

    # override to bypass changes in 2.7
    def getDescription(self, test):
        if self.descriptions:
            return test.shortDescription() or str(test)
        else:
            return str(test)

    def printLabel(self, label, err=None):
        # Might get patched into a streamless result
        stream = getattr(self, 'stream', None)
        if stream is not None:
            if self.showAll:
                message = [label]
                if err:
                    detail = _exception_detail(err[1])
                    if detail:
                        message.append(detail)
                stream.writeln(": ".join(message))
            elif self.dots:
                stream.write(label[:1])

    def printErrors(self):
        """Overrides to print all errorClasses errors as well.
        """
        TextTestResult.printErrors(self)
        for cls in self.errorClasses.keys():
            storage, label, isfail = self.errorClasses[cls]
            if isfail:
                self.printErrorList(label, storage)
        # Might get patched into a result with no config
        if hasattr(self, 'config'):
            self.config.plugins.report(self.stream)

    def printSummary(self, start, stop):
        """Called by the test runner to print the final summary of test
        run results.
        """
        write = self.stream.write
        writeln = self.stream.writeln
        taken = float(stop - start)
        run = self.testsRun
        plural = run != 1 and "s" or ""

        writeln(self.separator2)
        writeln("Ran %s test%s in %.3fs" % (run, plural, taken))
        writeln()

        summary = {}
        eckeys = self.errorClasses.keys()
        for cls in eckeys:
            storage, label, isfail = self.errorClasses[cls]
            count = len(storage)
            if not count:
                continue
            summary[label] = count
        if len(self.failures):
            summary['failures'] = len(self.failures)
        if len(self.errors):
            summary['errors'] = len(self.errors)

        if not self.wasSuccessful():
            write("FAILED")
        else:
            write("OK")
        items = summary.items()
        if items:
            sorted(items)
            write(" (")
            write(", ".join(["%s=%s" % (label, count) for
                             label, count in items]))
            writeln(")")
        else:
            writeln()

    def wasSuccessful(self):
        """Overrides to check that there are no errors in errorClasses
        lists that are marked as errors and should cause a run to
        fail.
        """
        if self.errors or self.failures:
            return False
        for cls in self.errorClasses.keys():
            storage, label, isfail = self.errorClasses[cls]
            if not isfail:
                continue
            if storage:
                return False
        return True

    def _addError(self, test, err):
        try:
            exc_info = self._exc_info_to_string(err, test)
        except TypeError:
            # 2.3: does not take test arg
            exc_info = self._exc_info_to_string(err)
        self.errors.append((test, exc_info))
        if self.showAll:
            self.stream.write('ERROR')
        elif self.dots:
            self.stream.write('E')

    def _exc_info_to_string(self, err, test=None):
        from nose.plugins.skip import SkipTest
        if isclass(err[0]) and issubclass(err[0], SkipTest):
            return str(err[1])
        try:
            return TextTestResult._exc_info_to_string(self, err, test)
        except TypeError:
            # 2.3: does not take test arg
            return TextTestResult._exc_info_to_string(self, err)


def ln(*arg, **kw):
    from warnings import warn
    warn("ln() has moved to nose.util from nose.result and will be removed "
         "from nose.result in a future release. Please update your imports ",
         DeprecationWarning)
    return _ln(*arg, **kw)



# make sure all tests in this file are dispatched to the same subprocess
def setup():
    pass


def test_timeout():
    "this test *should* fail when process-timeout=1"
    from time import sleep
    sleep(2)


# check timeout will not prevent remaining tests dispatched to the same subprocess to continue to run
def test_pass():
    pass



import os
import unittest

from nose.plugins import PluginTester
from nose.plugins.capture import Capture
from nose.plugins.failuredetail import FailureDetail

support = os.path.join(os.path.dirname(__file__), 'support')


class TestFailureDetail(PluginTester, unittest.TestCase):
    activate = "-d"
    args = ['-v']
    plugins = [FailureDetail()]
    suitepath = os.path.join(support, 'fdp')

    def runTest(self):
        print
        '*' * 70
        print
        str(self.output)
        print
        '*' * 70

        expect = \
            'AssertionError: a is not 4\n'
        '    print "Hello"\n'
        '    2 = 2\n'
        '>>  assert 2 == 4, "a is not 4"'

        assert expect in self.output


class TestFailureDetailWithCapture(PluginTester, unittest.TestCase):
    activate = "-d"
    args = ['-v']
    plugins = [FailureDetail(), Capture()]
    suitepath = os.path.join(support, 'fdp/test_fdp_no_capt.py')

    def runTest(self):
        print
        '*' * 70
        print
        str(self.output)
        print
        '*' * 70

        expect = \
            'AssertionError: a is not 4\n'
        '    print "Hello"\n'
        '    2 = 2\n'
        '>>  assert 2 == 4, "a is not 4"'

        assert expect in self.output


class TestFailureDetailWithUnicodeAndCapture(PluginTester, unittest.TestCase):
    activate = "-d"
    args = ['-v']
    plugins = [FailureDetail(), Capture()]
    suitepath = os.path.join(support, 'issue720')

    def runTest(self):
        print
        '*' * 70
        print
        str(self.output)
        print
        '*' * 70

        assert 'UnicodeDecodeError' not in self.output
        assert 'UnicodeEncodeError' not in self.output


if __name__ == '__main__':
    unittest.main()



import os
from unittest import TestCase

from nose.plugins import PluginTester
from nose.plugins.multiprocess import MultiProcess
from nose.plugins.skip import SkipTest

support = os.path.join(os.path.dirname(__file__), 'support')


def setup():
    try:
        import multiprocessing
        if 'active' in MultiProcess.status:
            raise SkipTest("Multiprocess plugin is active. Skipping tests of "
                           "plugin itself.")
    except ImportError:
        raise SkipTest("multiprocessing module not available")


class MPTestBase(PluginTester, TestCase):
    processes = 1
    activate = '--processes=1'
    plugins = [MultiProcess()]
    suitepath = os.path.join(support, 'timeout.py')

    def __init__(self, *args, **kwargs):
        self.activate = '--processes=%d' % self.processes
        PluginTester.__init__(self)
        TestCase.__init__(self, *args, **kwargs)



"""Exceptions for marking tests as skipped or deprecated.

This module exists to provide backwards compatibility with previous
versions of nose where skipped and deprecated tests were core
functionality, rather than being provided by plugins. It may be
removed in a future release.
"""



import os
import re
import sys
import unittest

from nose import main

from io import StringIO

support = os.path.join(os.path.dirname(__file__), 'support')

expected_output_pattern = """KeyError: '?fake'?\n-+ >> begin captured stdout << -+\nsomething\n"""


class TestIssue135(unittest.TestCase):
    def test_issue135(self):
        wd = os.path.join(support, 'issue135')
        stringio = StringIO()
        old_argv = sys.argv
        old_stderr = sys.stderr
        sys.stderr = stringio
        sys.argv = ["nosetests", wd]
        try:
            main(exit=False)
        finally:
            sys.argv = old_argv
            sys.stderr = old_stderr
        stringio.seek(0)
        output = stringio.read()
        self.assertTrue(re.search(expected_output_pattern, output) is not None)


if __name__ == '__main__':
    unittest.main()



import os
import unittest


class TestBug105(unittest.TestCase):

    def test_load_in_def_order(self):
        from nose.loader import TestLoader

        where = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             'support', 'bug105'))

        l = TestLoader()
        testmod = l.loadTestsFromDir(where).next()
        print(testmod)
        testmod.setUp()

        def fix(t):
            s = str(t)
            if ': ' in s:
                return s[s.index(': ') + 2:]
            return s

        tests = map(fix, testmod)
        print(tests)
        self.assertEqual(tests, ['tests.test_z', 'tests.test_a',
                                 'tests.test_dz', 'tests.test_mdz',
                                 'tests.test_b'])


if __name__ == '__main__':
    unittest.main()



"""Implements nose test program and collector.
"""
from __future__ import generators

import logging
import os
import sys
import time
import unittest

from nose.config import Config, all_config_files
from nose.loader import defaultTestLoader
from nose.plugins.manager import PluginManager, DefaultPluginManager, \
    RestrictedPluginManager
from nose.result import TextTestResult, NoseTextTestResult
from nose.suite import FinalizingSuiteWrapper
from nose.util import isclass, tolist

log = logging.getLogger('nose.core')
compat_24 = sys.version_info >= (2, 4)

__all__ = ['TestProgram', 'main', 'run', 'run_exit', 'runmodule', 'collector',
           'NoseTextTestRunner']


class NoseTextTestRunner(unittest.TextTestRunner):
    """Test runner that uses nose's TextTestResult to enable errorClasses,
    as well as providing hooks for plugins to override or replace the test
    output stream, results, and the test case itself.
    """

    def __init__(self, stream=sys.stderr, descriptions=1, verbosity=1,
                 config=None):
        if config is None:
            config = Config()
        self.config = config
        unittest.TextTestRunner.__init__(self, stream, descriptions, verbosity)

    def _makeResult(self):
        return NoseTextTestResult(self.stream,
                                  self.descriptions,
                                  self.verbosity,
                                  self.config)

    def run(self, test):
        """Overrides to provide plugin hooks and defer all output to
        the test result class.
        """
        wrapper = self.config.plugins.prepareTest(test)
        if wrapper is not None:
            test = wrapper

        # plugins can decorate or capture the output stream
        wrapped = self.config.plugins.setOutputStream(self.stream)
        if wrapped is not None:
            self.stream = wrapped

        result = self._makeResult()
        start = time.time()
        try:
            test(result)
        except KeyboardInterrupt:
            pass
        stop = time.time()
        result.printErrors()
        result.printSummary(start, stop)
        self.config.plugins.finalize(result)
        return result


class TestProgram(unittest.TestProgram):
    """Collect and run tests, returning success or failure.

    The arguments to TestProgram() are the same as to
    :func:`main()` and :func:`run()`:

    * module: All tests are in this module (default: None)
    * defaultTest: Tests to load (default: '.')
    * argv: Command line arguments (default: None; sys.argv is read)
    * testRunner: Test runner instance (default: None)
    * testLoader: Test loader instance (default: None)
    * env: Environment; ignored if config is provided (default: None;
      os.environ is read)
    * config: :class:`nose.config.Config` instance (default: None)
    * suite: Suite or list of tests to run (default: None). Passing a
      suite or lists of tests will bypass all test discovery and
      loading. *ALSO NOTE* that if you pass a unittest.TestSuite
      instance as the suite, context fixtures at the class, module and
      package level will not be used, and many plugin hooks will not
      be called. If you want normal nose behavior, either pass a list
      of tests, or a fully-configured :class:`nose.suite.ContextSuite`.
    * exit: Exit after running tests and printing report (default: True)
    * plugins: List of plugins to use; ignored if config is provided
      (default: load plugins with DefaultPluginManager)
    * addplugins: List of **extra** plugins to use. Pass a list of plugin
      instances in this argument to make custom plugins available while
      still using the DefaultPluginManager.
    """
    verbosity = 1

    def __init__(self, module=None, defaultTest='.', argv=None,
                 testRunner=None, testLoader=None, env=None, config=None,
                 suite=None, exit=True, plugins=None, addplugins=None):
        if env is None:
            env = os.environ
        if config is None:
            config = self.makeConfig(env, plugins)
        if addplugins:
            config.plugins.addPlugins(extraplugins=addplugins)
        self.config = config
        self.suite = suite
        self.exit = exit
        extra_args = {}
        version = sys.version_info[0:2]
        if version >= (2, 7) and version != (3, 0):
            extra_args['exit'] = exit
        unittest.TestProgram.__init__(
            self, module=module, defaultTest=defaultTest,
            argv=argv, testRunner=testRunner, testLoader=testLoader,
            **extra_args)

    def getAllConfigFiles(self, env=None):
        env = env or {}
        if env.get('NOSE_IGNORE_CONFIG_FILES', False):
            return []
        else:
            return all_config_files()

    def makeConfig(self, env, plugins=None):
        """Load a Config, pre-filled with user config files if any are
        found.
        """
        cfg_files = self.getAllConfigFiles(env)
        if plugins:
            manager = PluginManager(plugins=plugins)
        else:
            manager = DefaultPluginManager()
        return Config(
            env=env, files=cfg_files, plugins=manager)

    def parseArgs(self, argv):
        """Parse argv and env and configure running environment.
        """
        self.config.configure(argv, doc=self.usage())
        log.debug("configured %s", self.config)

        # quick outs: version, plugins (optparse would have already
        # caught and exited on help)
        if self.config.options.version:
            from nose import __version__
            sys.stdout = sys.__stdout__
            print("%s version %s" % (os.path.basename(sys.argv[0]), __version__))
            sys.exit(0)

        if self.config.options.showPlugins:
            self.showPlugins()
            sys.exit(0)

        if self.testLoader is None:
            self.testLoader = defaultTestLoader(config=self.config)
        elif isclass(self.testLoader):
            self.testLoader = self.testLoader(config=self.config)
        plug_loader = self.config.plugins.prepareTestLoader(self.testLoader)
        if plug_loader is not None:
            self.testLoader = plug_loader
        log.debug("test loader is %s", self.testLoader)

        # FIXME if self.module is a string, add it to self.testNames? not sure

        if self.config.testNames:
            self.testNames = self.config.testNames
        else:
            self.testNames = tolist(self.defaultTest)
        log.debug('defaultTest %s', self.defaultTest)
        log.debug('Test names are %s', self.testNames)
        if self.config.workingDir is not None:
            os.chdir(self.config.workingDir)
        self.createTests()

    def createTests(self):
        """Create the tests to run. If a self.suite
        is set, then that suite will be used. Otherwise, tests will be
        loaded from the given test names (self.testNames) using the
        test loader.
        """
        log.debug("createTests called with %s", self.suite)
        if self.suite is not None:
            # We were given an explicit suite to run. Make sure it's
            # loaded and wrapped correctly.
            self.test = self.testLoader.suiteClass(self.suite)
        else:
            self.test = self.testLoader.loadTestsFromNames(self.testNames)

    def runTests(self):
        """Run Tests. Returns true on success, false on failure, and sets
        self.success to the same value.
        """
        log.debug("runTests called")
        if self.testRunner is None:
            self.testRunner = NoseTextTestRunner(stream=self.config.stream,
                                                 verbosity=self.config.verbosity,
                                                 config=self.config)
        plug_runner = self.config.plugins.prepareTestRunner(self.testRunner)
        if plug_runner is not None:
            self.testRunner = plug_runner
        result = self.testRunner.run(self.test)
        self.success = result.wasSuccessful()
        if self.exit:
            sys.exit(not self.success)
        return self.success

    def showPlugins(self):
        """Print list of available plugins.
        """
        import textwrap

        class DummyParser:
            def __init__(self):
                self.options = []

            def add_option(self, *arg, **kw):
                self.options.append((arg, kw.pop('help', '')))

        v = self.config.verbosity
        self.config.plugins.sort()
        for p in self.config.plugins:
            print("Plugin %s" % p.name)
            if v >= 2:
                print("  score: %s" % p.score)
                print('\n'.join(textwrap.wrap(p.help().strip(),
                                              initial_indent='  ',
                                              subsequent_indent='  ')))
                if v >= 3:
                    parser = DummyParser()
                    p.addOptions(parser)
                    if len(parser.options):
                        print()
                        print("  Options:")
                        for opts, help in parser.options:
                            print('  %s' % (', '.join(opts)))
                            if help:
                                print('\n'.join(
                                    textwrap.wrap(help.strip(),
                                                  initial_indent='    ',
                                                  subsequent_indent='    ')))
                print()

    def usage(cls):
        import nose
        try:
            ld = nose.__loader__
            text = ld.get_data(os.path.join(
                os.path.dirname(__file__), 'usage.txt'))
        except AttributeError:
            f = open(os.path.join(
                os.path.dirname(__file__), 'usage.txt'), 'r')
            try:
                text = f.read()
            finally:
                f.close()
        # Ensure that we return str, not bytes.
        if not isinstance(text, str):
            text = text.decode('utf-8')
        return text

    usage = classmethod(usage)


# backwards compatibility
run_exit = main = TestProgram


def run(*arg, **kw):
    """Collect and run tests, returning success or failure.

    The arguments to `run()` are the same as to `main()`:

    * module: All tests are in this module (default: None)
    * defaultTest: Tests to load (default: '.')
    * argv: Command line arguments (default: None; sys.argv is read)
    * testRunner: Test runner instance (default: None)
    * testLoader: Test loader instance (default: None)
    * env: Environment; ignored if config is provided (default: None;
      os.environ is read)
    * config: :class:`nose.config.Config` instance (default: None)
    * suite: Suite or list of tests to run (default: None). Passing a
      suite or lists of tests will bypass all test discovery and
      loading. *ALSO NOTE* that if you pass a unittest.TestSuite
      instance as the suite, context fixtures at the class, module and
      package level will not be used, and many plugin hooks will not
      be called. If you want normal nose behavior, either pass a list
      of tests, or a fully-configured :class:`nose.suite.ContextSuite`.
    * plugins: List of plugins to use; ignored if config is provided
      (default: load plugins with DefaultPluginManager)
    * addplugins: List of **extra** plugins to use. Pass a list of plugin
      instances in this argument to make custom plugins available while
      still using the DefaultPluginManager.

    With the exception that the ``exit`` argument is always set
    to False.
    """
    kw['exit'] = False
    return TestProgram(*arg, **kw).success


def runmodule(name='__main__', **kw):
    """Collect and run tests in a single module only. Defaults to running
    tests in __main__. Additional arguments to TestProgram may be passed
    as keyword arguments.
    """
    main(defaultTest=name, **kw)


def collector():
    """TestSuite replacement entry point. Use anywhere you might use a
    unittest.TestSuite. The collector will, by default, load options from
    all config files and execute loader.loadTestsFromNames() on the
    configured testNames, or '.' if no testNames are configured.
    """
    # plugins that implement any of these methods are disabled, since
    # we don't control the test runner and won't be able to run them
    # finalize() is also not called, but plugins that use it aren't disabled,
    # because capture needs it.
    setuptools_incompat = ('report', 'prepareTest',
                           'prepareTestLoader', 'prepareTestRunner',
                           'setOutputStream')

    plugins = RestrictedPluginManager(exclude=setuptools_incompat)
    conf = Config(files=all_config_files(),
                  plugins=plugins)
    conf.configure(argv=['collector'])
    loader = defaultTestLoader(conf)

    if conf.testNames:
        suite = loader.loadTestsFromNames(conf.testNames)
    else:
        suite = loader.loadTestsFromNames(('.',))
    return FinalizingSuiteWrapper(suite, plugins.finalize)


if __name__ == '__main__':
    main()



"""
Test Loader
-----------

nose's test loader implements the same basic functionality as its
superclass, unittest.TestLoader, but extends it by more liberal
interpretations of what may be a test and how a test may be named.
"""
from __future__ import generators

import logging
import os
import sys
import types
import unittest
from inspect import isfunction

from nose.case import FunctionTestCase, MethodTestCase
from nose.config import Config
from nose.failure import Failure
from nose.importer import Importer, add_path, remove_path
from nose.pyversion import sort_list, cmp_to_key
from nose.pyversion import unbound_method, ismethod
from nose.selector import defaultSelector, TestAddress
from nose.suite import ContextSuiteFactory, ContextList, LazySuite
from nose.util import func_lineno, getpackage, isclass, isgenerator, \
    ispackage, regex_last_key, resolve_name, src, transplant_func, \
    transplant_class, test_address

log = logging.getLogger(__name__)
# log.setLevel(logging.DEBUG)

# for efficiency and easier mocking
op_normpath = os.path.normpath
op_abspath = os.path.abspath
op_join = os.path.join
op_isdir = os.path.isdir
op_isfile = os.path.isfile

__all__ = ['TestLoader', 'defaultTestLoader']


class TestLoader(unittest.TestLoader):
    """Test loader that extends unittest.TestLoader to:

    * Load tests from test-like functions and classes that are not
      unittest.TestCase subclasses
    * Find and load test modules in a directory
    * Support tests that are generators
    * Support easy extensions of or changes to that behavior through plugins
    """
    config = None
    importer = None
    workingDir = None
    selector = None
    suiteClass = None

    def __init__(self, config=None, importer=None, workingDir=None,
                 selector=None):
        """Initialize a test loader.

        Parameters (all optional):

        * config: provide a `nose.config.Config`_ or other config class
          instance; if not provided a `nose.config.Config`_ with
          default values is used.
        * importer: provide an importer instance that implements
          `importFromPath`. If not provided, a
          `nose.importer.Importer`_ is used.
        * workingDir: the directory to which file and module names are
          relative. If not provided, assumed to be the current working
          directory.
        * selector: a selector class or instance. If a class is
          provided, it will be instantiated with one argument, the
          current config. If not provided, a `nose.selector.Selector`_
          is used.
        """
        if config is None:
            config = Config()
        if importer is None:
            importer = Importer(config=config)
        if workingDir is None:
            workingDir = config.workingDir
        if selector is None:
            selector = defaultSelector(config)
        elif isclass(selector):
            selector = selector(config)
        self.config = config
        self.importer = importer
        self.workingDir = op_normpath(op_abspath(workingDir))
        self.selector = selector
        if config.addPaths:
            add_path(workingDir, config)
        self.suiteClass = ContextSuiteFactory(config=config)

        self._visitedPaths = set([])

        unittest.TestLoader.__init__(self)

    def getTestCaseNames(self, testCaseClass):
        """Override to select with selector, unless
        config.getTestCaseNamesCompat is True
        """
        if self.config.getTestCaseNamesCompat:
            return unittest.TestLoader.getTestCaseNames(self, testCaseClass)

        def wanted(attr, cls=testCaseClass, sel=self.selector):
            item = getattr(cls, attr, None)
            if isfunction(item):
                item = unbound_method(cls, item)
            elif not ismethod(item):
                return False
            return sel.wantMethod(item)

        cases = list(filter(wanted, dir(testCaseClass)))

        # add runTest if nothing else picked
        if not cases and hasattr(testCaseClass, 'runTest'):
            cases = ['runTest']
        if self.sortTestMethodsUsing:
            cases = sort_list(cases, cmp_to_key(self.sortTestMethodsUsing))
        return cases

    def _haveVisited(self, path):
        # For cases where path is None, we always pretend we haven't visited
        # them.
        if path is None:
            return False

        return path in self._visitedPaths

    def _addVisitedPath(self, path):
        if path is not None:
            self._visitedPaths.add(path)

    def loadTestsFromDir(self, path):
        """Load tests from the directory at path. This is a generator
        -- each suite of tests from a module or other file is yielded
        and is expected to be executed before the next file is
        examined.
        """
        log.debug("load from dir %s", path)
        plugins = self.config.plugins
        plugins.beforeDirectory(path)
        if self.config.addPaths:
            paths_added = add_path(path, self.config)

        entries = os.listdir(path)
        entries = sort_list(entries, regex_last_key(self.config.testMatch))
        for entry in entries:
            # this hard-coded initial-dot test will be removed:
            # http://code.google.com/p/python-nose/issues/detail?id=82
            if entry.startswith('.'):
                continue
            if src(entry) == '__init__.py':
                continue
            entry_path = op_abspath(op_join(path, entry))
            is_file = op_isfile(entry_path)
            wanted = False
            if is_file:
                is_dir = False
                wanted = self.selector.wantFile(entry_path)
            else:
                is_dir = op_isdir(entry_path)
                if is_dir:
                    # this hard-coded initial-underscore test will be removed:
                    # http://code.google.com/p/python-nose/issues/detail?id=82
                    if entry.startswith('_'):
                        continue
                    wanted = self.selector.wantDirectory(entry_path)
            is_package = ispackage(entry_path)

            # Python 3.3 now implements PEP 420: Implicit Namespace Packages.
            # As a result, it's now possible that parent paths that have a
            # segment with the same basename as our package ends up
            # in module.__path__.  So we have to keep track of what we've
            # visited, and not-revisit them again.
            if wanted and not self._haveVisited(entry_path):
                self._addVisitedPath(entry_path)
                if is_file:
                    plugins.beforeContext()
                    if entry.endswith('.py'):
                        yield self.loadTestsFromName(
                            entry_path, discovered=True)
                    else:
                        yield self.loadTestsFromFile(entry_path)
                    plugins.afterContext()
                elif is_package:
                    # Load the entry as a package: given the full path,
                    # loadTestsFromName() will figure it out
                    yield self.loadTestsFromName(
                        entry_path, discovered=True)
                else:
                    # Another test dir in this one: recurse lazily
                    yield self.suiteClass(
                        lambda: self.loadTestsFromDir(entry_path))
        tests = []
        for test in plugins.loadTestsFromDir(path):
            tests.append(test)
        # TODO: is this try/except needed?
        try:
            if tests:
                yield self.suiteClass(tests)
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            yield self.suiteClass([Failure(*sys.exc_info())])

        # pop paths
        if self.config.addPaths:
            for p in paths_added:
                remove_path(p)
        plugins.afterDirectory(path)

    def loadTestsFromFile(self, filename):
        """Load tests from a non-module file. Default is to raise a
        ValueError; plugins may implement `loadTestsFromFile` to
        provide a list of tests loaded from the file.
        """
        log.debug("Load from non-module file %s", filename)
        try:
            tests = [test for test in
                     self.config.plugins.loadTestsFromFile(filename)]
            if tests:
                # Plugins can yield False to indicate that they were
                # unable to load tests from a file, but it was not an
                # error -- the file just had no tests to load.
                tests = list(filter(None, tests))
                return self.suiteClass(tests)
            else:
                # Nothing was able to even try to load from this file
                open(filename, 'r').close()  # trigger os error
                raise ValueError("Unable to load tests from file %s"
                                 % filename)
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            exc = sys.exc_info()
            return self.suiteClass(
                [Failure(exc[0], exc[1], exc[2],
                         address=(filename, None, None))])

    def loadTestsFromGenerator(self, generator, module):
        """Lazy-load tests from a generator function. The generator function
        may yield either:

        * a callable, or
        * a function name resolvable within the same module
        """

        def generate(g=generator, m=module):
            try:
                for test in g():
                    test_func, arg = self.parseGeneratedTest(test)
                    if not callable(test_func):
                        test_func = getattr(m, test_func)
                    yield FunctionTestCase(test_func, arg=arg, descriptor=g)
            except KeyboardInterrupt:
                raise
            except Exception:
                exc = sys.exc_info()
                yield Failure(exc[0], exc[1], exc[2],
                              address=test_address(generator))

        return self.suiteClass(generate, context=generator, can_split=False)

    def loadTestsFromGeneratorMethod(self, generator, cls):
        """Lazy-load tests from a generator method.

        This is more complicated than loading from a generator function,
        since a generator method may yield:

        * a function
        * a bound or unbound method, or
        * a method name
        """
        # convert the unbound generator method
        # into a bound method so it can be called below
        if hasattr(generator, '__self__'):
            cls = generator.__self__.__class__
        inst = cls()
        method = generator.__name__
        generator = getattr(inst, method)

        def generate(g=generator, c=cls):
            try:
                for test in g():
                    test_func, arg = self.parseGeneratedTest(test)
                    if not callable(test_func):
                        test_func = unbound_method(c, getattr(c, test_func))
                    if ismethod(test_func):
                        yield MethodTestCase(test_func, arg=arg, descriptor=g)
                    elif callable(test_func):
                        # In this case we're forcing the 'MethodTestCase'
                        # to run the inline function as its test call,
                        # but using the generator method as the 'method of
                        # record' (so no need to pass it as the descriptor)
                        yield MethodTestCase(g, test=test_func, arg=arg)
                    else:
                        yield Failure(
                            TypeError,
                            "%s is not a callable or method" % test_func)
            except KeyboardInterrupt:
                raise
            except Exception:
                exc = sys.exc_info()
                yield Failure(exc[0], exc[1], exc[2],
                              address=test_address(generator))

        return self.suiteClass(generate, context=generator, can_split=False)

    def loadTestsFromModule(self, module, path=None, discovered=False):
        """Load all tests from module and return a suite containing
        them. If the module has been discovered and is not test-like,
        the suite will be empty by default, though plugins may add
        their own tests.
        """
        log.debug("Load from module %s", module)
        tests = []
        test_classes = []
        test_funcs = []
        # For *discovered* modules, we only load tests when the module looks
        # testlike. For modules we've been directed to load, we always
        # look for tests. (discovered is set to True by loadTestsFromDir)
        if not discovered or self.selector.wantModule(module):
            for item in dir(module):
                test = getattr(module, item, None)
                if isclass(test):
                    if self.selector.wantClass(test):
                        test_classes.append(test)
                elif isfunction(test) and self.selector.wantFunction(test):
                    test_funcs.append(test)

            test_classes = sort_list(test_classes, lambda x: x.__name__)
            tests.extend(self.makeTest(t, parent=module) for t in test_classes)

            test_funcs = sort_list(test_funcs, func_lineno)
            tests.extend(self.makeTest(t, parent=module) for t in test_funcs)

        # Now, descend into packages
        # FIXME can or should this be lazy?
        # is this syntax 2.2 compatible?
        module_paths = getattr(module, '__path__', [])

        if path:
            path = os.path.normcase(os.path.realpath(path))

        for module_path in module_paths:
            log.debug("Load tests from module path %s?", module_path)
            log.debug("path: %s os.path.realpath(%s): %s",
                      path, os.path.normcase(module_path),
                      os.path.realpath(os.path.normcase(module_path)))
            if (self.config.traverseNamespace or not path) or \
                    os.path.realpath(
                        os.path.normcase(module_path)).startswith(path):
                # Egg files can be on sys.path, so make sure the path is a
                # directory before trying to load from it.
                if os.path.isdir(module_path):
                    tests.extend(self.loadTestsFromDir(module_path))

        for test in self.config.plugins.loadTestsFromModule(module, path):
            tests.append(test)

        return self.suiteClass(ContextList(tests, context=module))

    def loadTestsFromName(self, name, module=None, discovered=False):
        """Load tests from the entity with the given name.

        The name may indicate a file, directory, module, or any object
        within a module. See `nose.util.split_test_name` for details on
        test name parsing.
        """
        # FIXME refactor this method into little bites?
        log.debug("load from %s (%s)", name, module)

        suite = self.suiteClass

        # give plugins first crack
        plug_tests = self.config.plugins.loadTestsFromName(name, module)
        if plug_tests:
            return suite(plug_tests)

        addr = TestAddress(name, workingDir=self.workingDir)
        if module:
            # Two cases:
            #  name is class.foo
            #    The addr will be incorrect, since it thinks class.foo is
            #    a dotted module name. It's actually a dotted attribute
            #    name. In this case we want to use the full submitted
            #    name as the name to load from the module.
            #  name is module:class.foo
            #    The addr will be correct. The part we want is the part after
            #    the :, which is in addr.call.
            if addr.call:
                name = addr.call
            parent, obj = self.resolve(name, module)
            if (isclass(parent)
                    and getattr(parent, '__module__', None) != module.__name__
                    and not isinstance(obj, Failure)):
                parent = transplant_class(parent, module.__name__)
                obj = getattr(parent, obj.__name__)
            log.debug("parent %s obj %s module %s", parent, obj, module)
            if isinstance(obj, Failure):
                return suite([obj])
            else:
                return suite(ContextList([self.makeTest(obj, parent)],
                                         context=parent))
        else:
            if addr.module:
                try:
                    if addr.filename is None:
                        module = resolve_name(addr.module)
                    else:
                        self.config.plugins.beforeImport(
                            addr.filename, addr.module)
                        # FIXME: to support module.name names,
                        # do what resolve-name does and keep trying to
                        # import, popping tail of module into addr.call,
                        # until we either get an import or run out of
                        # module parts
                        try:
                            module = self.importer.importFromPath(
                                addr.filename, addr.module)
                        finally:
                            self.config.plugins.afterImport(
                                addr.filename, addr.module)
                except (KeyboardInterrupt, SystemExit):
                    raise
                except Exception:
                    exc = sys.exc_info()
                    return suite([Failure(exc[0], exc[1], exc[2],
                                          address=addr.totuple())])
                if addr.call:
                    return self.loadTestsFromName(addr.call, module)
                else:
                    return self.loadTestsFromModule(
                        module, addr.filename,
                        discovered=discovered)
            elif addr.filename:
                path = addr.filename
                if addr.call:
                    package = getpackage(path)
                    if package is None:
                        return suite([
                            Failure(ValueError,
                                    "Can't find callable %s in file %s: "
                                    "file is not a python module" %
                                    (addr.call, path),
                                    address=addr.totuple())])
                    return self.loadTestsFromName(addr.call, module=package)
                else:
                    if op_isdir(path):
                        # In this case we *can* be lazy since we know
                        # that each module in the dir will be fully
                        # loaded before its tests are executed; we
                        # also know that we're not going to be asked
                        # to load from . and ./some_module.py *as part
                        # of this named test load*
                        return LazySuite(
                            lambda: self.loadTestsFromDir(path))
                    elif op_isfile(path):
                        return self.loadTestsFromFile(path)
                    else:
                        return suite([
                            Failure(OSError, "No such file %s" % path,
                                    address=addr.totuple())])
            else:
                # just a function? what to do? I think it can only be
                # handled when module is not None
                return suite([
                    Failure(ValueError, "Unresolvable test name %s" % name,
                            address=addr.totuple())])

    def loadTestsFromNames(self, names, module=None):
        """Load tests from all names, returning a suite containing all
        tests.
        """
        plug_res = self.config.plugins.loadTestsFromNames(names, module)
        if plug_res:
            suite, names = plug_res
            if suite:
                return self.suiteClass([
                    self.suiteClass(suite),
                    unittest.TestLoader.loadTestsFromNames(self, names, module)
                ])
        return unittest.TestLoader.loadTestsFromNames(self, names, module)

    def loadTestsFromTestCase(self, testCaseClass):
        """Load tests from a unittest.TestCase subclass.
        """
        cases = []
        plugins = self.config.plugins
        for case in plugins.loadTestsFromTestCase(testCaseClass):
            cases.append(case)
        # For efficiency in the most common case, just call and return from
        # super. This avoids having to extract cases and rebuild a context
        # suite when there are no plugin-contributed cases.
        if not cases:
            return super(TestLoader, self).loadTestsFromTestCase(testCaseClass)
        cases.extend(
            [case for case in
             super(TestLoader, self).loadTestsFromTestCase(testCaseClass)])
        return self.suiteClass(cases)

    def loadTestsFromTestClass(self, cls):
        """Load tests from a test class that is *not* a unittest.TestCase
        subclass.

        In this case, we can't depend on the class's `__init__` taking method
        name arguments, so we have to compose a MethodTestCase for each
        method in the class that looks testlike.
        """

        def wanted(attr, cls=cls, sel=self.selector):
            item = getattr(cls, attr, None)
            if isfunction(item):
                item = unbound_method(cls, item)
            elif not ismethod(item):
                return False
            return sel.wantMethod(item)

        cases = [self.makeTest(getattr(cls, case), cls)
                 for case in filter(wanted, dir(cls))]
        for test in self.config.plugins.loadTestsFromTestClass(cls):
            cases.append(test)
        return self.suiteClass(ContextList(cases, context=cls))

    def makeTest(self, obj, parent=None):
        try:
            return self._makeTest(obj, parent)
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            exc = sys.exc_info()
            try:
                addr = test_address(obj)
            except KeyboardInterrupt:
                raise
            except Exception:
                addr = None
            return Failure(exc[0], exc[1], exc[2], address=addr)

    def _makeTest(self, obj, parent=None):
        """Given a test object and its parent, return a test case
        or test suite.
        """
        plug_tests = []
        try:
            addr = test_address(obj)
        except KeyboardInterrupt:
            raise
        except Exception:
            addr = None
        for test in self.config.plugins.makeTest(obj, parent):
            plug_tests.append(test)
        # TODO: is this try/except needed?
        try:
            if plug_tests:
                return self.suiteClass(plug_tests)
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            exc = sys.exc_info()
            return Failure(exc[0], exc[1], exc[2], address=addr)

        if isfunction(obj) and parent and not isinstance(parent, types.ModuleType):
            # This is a Python 3.x 'unbound method'.  Wrap it with its
            # associated class..
            obj = unbound_method(parent, obj)

        if isinstance(obj, unittest.TestCase):
            return obj
        elif isclass(obj):
            if parent and obj.__module__ != parent.__name__:
                obj = transplant_class(obj, parent.__name__)
            if issubclass(obj, unittest.TestCase):
                return self.loadTestsFromTestCase(obj)
            else:
                return self.loadTestsFromTestClass(obj)
        elif ismethod(obj):
            if parent is None:
                parent = obj.__class__
            if issubclass(parent, unittest.TestCase):
                return parent(obj.__name__)
            else:
                if isgenerator(obj):
                    return self.loadTestsFromGeneratorMethod(obj, parent)
                else:
                    return MethodTestCase(obj)
        elif isfunction(obj):
            if parent and obj.__module__ != parent.__name__:
                obj = transplant_func(obj, parent.__name__)
            if isgenerator(obj):
                return self.loadTestsFromGenerator(obj, parent)
            else:
                return FunctionTestCase(obj)
        else:
            return Failure(TypeError,
                           "Can't make a test from %s" % obj,
                           address=addr)

    def resolve(self, name, module):
        """Resolve name within module
        """
        obj = module
        parts = name.split('.')
        for part in parts:
            parent, obj = obj, getattr(obj, part, None)
        if obj is None:
            # no such test
            obj = Failure(ValueError, "No such test %s" % name)
        return parent, obj

    def parseGeneratedTest(self, test):
        """Given the yield value of a test generator, return a func and args.

        This is used in the two loadTestsFromGenerator* methods.

        """
        if not isinstance(test, tuple):  # yield test
            test_func, arg = (test, tuple())
        elif len(test) == 1:  # yield (test,)
            test_func, arg = (test[0], tuple())
        else:  # yield test, foo, bar, ...
            assert len(test) > 1  # sanity check
            test_func, arg = (test[0], test[1:])
        return test_func, arg


defaultTestLoader = TestLoader



called = []


def globs(globs):
    globs['something'] = 'Something?'
    return globs


def setup_module(module):
    module.called[:] = []


def setup_test(test):
    called.append(test)
    test.globs['count'] = len(called)


setup_test.__test__ = False


def teardown_test(test):
    pass


teardown_test.__test__ = False



import time
import unittest

from nose.tools import *


class TestTools(unittest.TestCase):

    def test_ok(self):
        ok_(True)
        try:
            ok_(False, "message")
        except AssertionError as e:
            assert str(e) == "message"
        else:
            self.fail("ok_(False) did not raise assertion error")

    def test_eq(self):
        eq_(1, 1)
        try:
            eq_(1, 0, "message")
        except AssertionError as e:
            assert str(e) == "message"
        else:
            self.fail("eq_(1, 0) did not raise assertion error")
        try:
            eq_(1, 0)
        except AssertionError as e:
            assert str(e) == "1 != 0"
        else:
            self.fail("eq_(1, 0) did not raise assertion error")

    def test_eq_unittest_flag(self):
        """Make sure eq_() is in a namespace that has __unittest = 1.

        This lets tracebacks refrain from descending into the eq_ frame.

        """
        assert '__unittest' in eq_

    def test_istest_unittest_flag(self):
        """Make sure istest() is not in a namespace that has __unittest = 1.

        That is, make sure our __unittest labeling didn't get overzealous.

        """
        assert '__unittest' not in istest.func_globals

    def test_raises(self):
        from nose.case import FunctionTestCase

        def raise_typeerror():
            raise TypeError("foo")

        def noraise():
            pass

        raise_good = raises(TypeError)(raise_typeerror)
        raise_other = raises(ValueError)(raise_typeerror)
        no_raise = raises(TypeError)(noraise)

        tc = FunctionTestCase(raise_good)
        self.assertEqual(str(tc), "%s.%s" % (__name__, 'raise_typeerror'))

        raise_good()
        try:
            raise_other()
        except TypeError as e:
            pass
        else:
            self.fail("raises did pass through unwanted exception")

        try:
            no_raise()
        except AssertionError as e:
            pass
        else:
            self.fail("raises did not raise assertion error on no exception")

    def test_timed(self):

        def too_slow():
            time.sleep(.3)

        too_slow = timed(.2)(too_slow)

        def quick():
            time.sleep(.1)

        quick = timed(.2)(quick)

        def check_result():
            return 42

        check_result = timed(.2)(check_result)

        assert 42 == check_result()

        quick()
        try:
            too_slow()
        except TimeExpired:
            pass
        else:
            self.fail("Slow test did not throw TimeExpired")

    def test_make_decorator(self):
        def func():
            pass

        func.setup = 'setup'
        func.teardown = 'teardown'

        def f1():
            pass

        f2 = make_decorator(func)(f1)

        assert f2.setup == 'setup'
        assert f2.teardown == 'teardown'

    def test_nested_decorators(self):
        from nose.tools import raises, timed, with_setup

        def test():
            pass

        def foo():
            pass

        test = with_setup(foo, foo)(test)
        test = timed(1.0)(test)
        test = raises(TypeError)(test)
        assert test.setup == foo
        assert test.teardown == foo

    def test_decorator_func_sorting(self):
        from nose.tools import raises, timed, with_setup
        from nose.util import func_lineno

        def test1():
            pass

        def test2():
            pass

        def test3():
            pass

        def foo():
            pass

        test1_pos = func_lineno(test1)
        test2_pos = func_lineno(test2)
        test3_pos = func_lineno(test3)

        test1 = raises(TypeError)(test1)
        test2 = timed(1.0)(test2)
        test3 = with_setup(foo)(test3)

        self.assertEqual(func_lineno(test1), test1_pos)
        self.assertEqual(func_lineno(test2), test2_pos)
        self.assertEqual(func_lineno(test3), test3_pos)

    def test_testcase_funcs(self):
        import nose.tools
        tc_asserts = [at for at in dir(nose.tools)
                      if at.startswith('assert_')]
        print(tc_asserts)

        # FIXME: not sure which of these are in all supported
        # versions of python
        assert 'assert_raises' in tc_asserts

    def test_multiple_with_setup(self):
        from nose.tools import with_setup
        from nose.case import FunctionTestCase
        from unittest import TestResult

        called = []

        def test():
            called.append('test')

        def test2():
            called.append('test2')

        def test3():
            called.append('test3')

        def s1():
            called.append('s1')

        def s2():
            called.append('s2')

        def s3():
            called.append('s3')

        def t1():
            called.append('t1')

        def t2():
            called.append('t2')

        def t3():
            called.append('t3')

        ws1 = with_setup(s1, t1)(test)
        case1 = FunctionTestCase(ws1)
        case1(TestResult())
        self.assertEqual(called, ['s1', 'test', 't1'])

        called[:] = []
        ws2 = with_setup(s2, t2)(test2)
        ws2 = with_setup(s1, t1)(ws2)
        case2 = FunctionTestCase(ws2)
        case2(TestResult())
        self.assertEqual(called, ['s1', 's2', 'test2', 't2', 't1'])

        called[:] = []
        ws3 = with_setup(s3, t3)(test3)
        ws3 = with_setup(s2, t2)(ws3)
        ws3 = with_setup(s1, t1)(ws3)
        case3 = FunctionTestCase(ws3)
        case3(TestResult())
        self.assertEqual(called, ['s1', 's2', 's3',
                                  'test3', 't3', 't2', 't1'])


if __name__ == '__main__':
    unittest.main()



import logging
import os
import re
import shutil
import sys

import setuptools.command.test
from setuptools import Command
from setuptools import Distribution as _Distribution
from setuptools import setup as _setup

import patch

py3_args = ['test_dirs', 'test_build_dir', 'doctest_exts', 'pyversion_patching']
patch.logger.setLevel(logging.WARN)
patchfile_re = re.compile(r'(.*)\.py([0-9.]+)\.patch$')


def pyversion_patch(filename):
    """
    :param filename:
    :return:
    Find the best pyversion-fixup patch for a given filename and apply it.
    """
    dir, file = os.path.split(filename)
    best_ver = (0,)
    patchfile = None
    for dirfile in os.listdir(dir):
        m = patchfile_re.match(dirfile)
        if not m:
            continue
        base, ver = m.groups()
        if base != file:
            continue
        ver = tuple([int(v) for v in ver.split('.')])
        if sys.version_info >= ver and ver > best_ver:
            best_ver = ver
            patchfile = dirfile
    if not patchfile:
        return False
    logging.info("Applying %s to %s..." % (patchfile, filename))
    cwd = os.getcwd()
    os.chdir(dir)
    try:
        p = patch.fromfile(patchfile)
        p.apply()
    finally:
        os.chdir(cwd)
    return True


class Distribution(_Distribution):
    def __init__(self, attrs=None):
        self.test_dirs = []
        self.test_build_dir = None
        self.doctest_exts = ['.py', '.rst']
        self.pyversion_patching = False
        _Distribution.__init__(self, attrs)


class BuildTestsCommand(Command):
    user_options = []

    def initialize_options(self):
        self.test_base = None

    def finalize_options(self):
        test_base = self.distribution.test_build_dir
        if not test_base:
            bcmd = self.get_finalized_command('build')
            test_base = bcmd.build_base
        self.test_base = test_base

    def run(self):
        test_dirs = getattr(self.distribution, 'test_dirs', [])
        test_base = self.test_base
        bpy_cmd = self.get_finalized_command("build_py")
        lib_base = os.path.normpath(bpy_cmd.build_lib)
        modified = []
        py_modified = []
        doc_modified = []
        os.makedirs(test_base)
        for testdir in test_dirs:
            for srcdir, dirnames, filenames in os.walk(testdir):
                destdir = os.path.join(test_base, srcdir)
                os.makedirs(destdir)
                for fn in filenames:
                    if fn.startswith("."):
                        # Skip .svn folders and such
                        continue
                    dstfile, copied = shutil.copyfile(
                        os.path.join(srcdir, fn),
                        os.path.join(destdir, fn))
                    if copied:
                        modified.append(dstfile)
                        if fn.endswith('.py'):
                            py_modified.append(dstfile)
                        for ext in self.distribution.doctest_exts:
                            if fn.endswith(ext):
                                doc_modified.append(dstfile)
                                break

        if self.distribution.pyversion_patching:
            if patch is not None:
                for file in modified:
                    pyversion_patch(file)
            else:
                logging.warn(
                    "Warning: pyversion_patching specified in setup config but patch module not found.  Patching "
                    "will not be performed.")

        os.makedirs(lib_base)
        self.reinitialize_command('egg_info', egg_base=lib_base)
        self.run_command('egg_info')


class TestCommand(setuptools.command.test.test):
    # Override 'test' command to make sure 'build_tests' gets run first.
    def run(self):
        self.run_command('build_tests')
        setuptools.command.test.test.run(self)


def setup(*args, **kwargs):
    kwargs.setdefault('distclass', Distribution)
    cmdclass = kwargs.setdefault('cmdclass', {})
    cmdclass.setdefault('build_tests', BuildTestsCommand)
    cmdclass.setdefault('test', TestCommand)
    return _setup(*args, **kwargs)



import os

from test_multiprocessing import MPTestBase


# test case for #462
class TestClassFixture(MPTestBase):
    suitepath = os.path.join(os.path.dirname(__file__), 'support', 'class.py')

    def runTest(self):
        assert str(self.output).strip().endswith('OK')
        assert 'Ran 2 tests' in self.output



import testlib
from mypackage import math


class TestBasicMath(testlib.Base):

    def test_add(self):
        self.assertEqual(math.add(1, 2), 3)

    def test_sub(self):
        self.assertEqual(math.sub(3, 1), 2)


class TestHelperClass:
    def __init__(self):
        raise Exception(
            "This test helper class should not be collected")



import os
import sys

VERSION = '1.7.0'
py_vers_tag = '-%s.%s' % sys.version_info[:2]

test_dirs = ['functional_tests', 'unit_tests', os.path.join('doc', 'doc_tests'), 'nose']

if sys.version_info >= (3, 12):
    try:
        import setuptools
    except ImportError:
        import setuptools
        from distribute_setup import use_setuptools

        use_setuptools()

    extra = {'test_dirs': test_dirs,
             'test_build_dir': 'build/tests',
             'pyversion_patching': True,
             }
else:
    extra = {}

try:
    from setup3lib import setup
    from setuptools import find_packages

    addl_args = dict(
        zip_safe=False,
        packages=find_packages(),
        entry_points={
            'console_scripts': [
                'nosetests = nose:run_exit',
                'nosetests%s = nose:run_exit' % py_vers_tag,
            ],
            'setuptools.commands': [
                ' nosetests = nose.commands:nosetests',
            ],
        },
        test_suite='nose.collector',
    )
    addl_args.update(extra)

    # This is required by multiprocess plugin; on Windows, if
    # the launch script is not import-safe, spawned processes
    # will re-run it, resulting in an infinite loop.
    if sys.platform == 'win32':
        import re
        from setuptools.command.easy_install import easy_install


        def wrap_write_script(self, script_name, contents, *arg, **kwarg):
            if script_name.endswith('.exe'):
                return self._write_script(script_name, contents, *arg, **kwarg)

            bad_text = re.compile(
                "\n"
                "sys.exit\(\n"
                "   load_entry_point\(([^\)]+)\)\(\)\n"
                "\)\n")
            good_text = (
                "\n"
                "if __name__ == '__main__':\n"
                "    sys.exit(\n"
                r"        load_entry_point(\1)()\n"
                "    )\n"
            )
            contents = bad_text.sub(good_text, contents)
            return self._write_script(script_name, contents, *arg, **kwarg)


        easy_install._write_script = easy_install.write_script
        easy_install.write_script = wrap_write_script

except ImportError:
    import re
    from setuptools.command.easy_install import easy_install
    from setuptools import setup
    from setup3lib import setup
    from setuptools import find_packages

    addl_args = dict(
        packages=['nose', 'nose.ext', 'nose.plugins', 'nose.sphinx',
                  'nose.tools'],
        scripts=['bin/nosetests'],
    )

setup(
    name='nose-py3',
    version=VERSION,
    author='Adam Bilbrough',
    author_email='',
    maintainer='Adam Bilbrough',
    install_requires=[
        'coverage',
        'six',
        'sphinx',
        'setuptools'
        ],
    description='nose extends unittest to make testing easier - python3 version',
    long_description=
    """nose extends the test loading and running features of unittest, making
        it easier to write, find and run tests.

        By default, nose will run tests in files or directories under the current
        working directory whose names include "test" or "Test" at a word boundary
        (like "test_this" or "functional_test" or "TestClass" but not
        "libtest"). Test output is similar to that of unittest, but also includes
        captured stdout output from failing tests, for easy print-style debugging.

        These features, and many more, are customizable through the use of
        plugins. Plugins included with nose provide support for doctest, code
        coverage and profiling, flexible attribute-based test selection,
        output capture and more. More information about writing plugins may be
        found on in the nose API documentation, here:
        http://readthedocs.org/docs/nose/

        If you have recently reported a bug marked as fixed, or have a craving for
        the very latest, you may want the development version instead:
        https://github.com/atsb/nose-py3
        """,
    license='GNU LGPL',
    keywords='test unittest doctest automatic discovery',
    url='https://github.com/atsb/nose-py3',
    data_files=[('man/man1', ['nosetests.1'])],
    package_data={'': ['*.txt',
                       'examples/*.py',
                       'examples/*/*.py']},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Testing'
    ],
    **addl_args
)



"""Simple traceback introspection. Used to add additional information to
AssertionErrors in tests, so that failure messages may be more informative.
"""
import inspect
import logging
import re
import sys
import textwrap
import tokenize

from io import StringIO

log = logging.getLogger(__name__)


def inspect_traceback(tb):
    """Inspect a traceback and its frame, returning source for the expression
    where the exception was raised, with simple variable replacement performed
    and the line on which the exception was raised marked with '>>'
    """
    log.debug('inspect traceback %s', tb)

    # we only want the innermost frame, where the exception was raised
    while tb.tb_next:
        tb = tb.tb_next

    frame = tb.tb_frame
    lines, exc_line = tbsource(tb)

    # figure out the set of lines to grab.
    inspect_lines, mark_line = find_inspectable_lines(lines, exc_line)
    src = StringIO(textwrap.dedent(''.join(inspect_lines)))
    exp = Expander(frame.f_locals, frame.f_globals)

    while inspect_lines:
        try:
            for tok in tokenize.generate_tokens(src.readline):
                exp(*tok)
        except tokenize.TokenError as e:
            # this can happen if our inspectable region happens to butt up
            # against the end of a construct like a docstring with the closing
            # """ on separate line
            log.debug("Tokenizer error: %s", e)
            inspect_lines.pop(0)
            mark_line -= 1
            src = StringIO(textwrap.dedent(''.join(inspect_lines)))
            exp = Expander(frame.f_locals, frame.f_globals)
            continue
        break
    padded = []
    if exp.expanded_source:
        exp_lines = exp.expanded_source.split('\n')
        ep = 0
        for line in exp_lines:
            if ep == mark_line:
                padded.append('>>  ' + line)
            else:
                padded.append('    ' + line)
            ep += 1
    return '\n'.join(padded)


def tbsource(tb, context=6):
    """Get source from  a traceback object.

    A tuple of two things is returned: a list of lines of context from
    the source code, and the index of the current line within that list.
    The optional second argument specifies the number of lines of context
    to return, which are centered around the current line.

    .. Note ::
       This is adapted from inspect.py in the python 2.4 standard library, 
       since a bug in the 2.3 version of inspect prevents it from correctly
       locating source lines in a traceback frame.
    """

    lineno = tb.tb_lineno
    frame = tb.tb_frame

    if context > 0:
        start = lineno - 1 - context // 2
        log.debug("lineno: %s start: %s", lineno, start)

        try:
            lines, dummy = inspect.findsource(frame)
        except IOError:
            lines, index = [''], 0
        else:
            all_lines = lines
            start = max(start, 1)
            start = max(0, min(start, len(lines) - context))
            lines = lines[start:start + context]
            index = lineno - 1 - start

            # python 2.5 compat: if previous line ends in a continuation,
            # decrement start by 1 to match 2.4 behavior                
            if sys.version_info >= (2, 5) and index > 0:
                while lines[index - 1].strip().endswith('\\'):
                    start -= 1
                    lines = all_lines[start:start + context]
    else:
        lines, index = [''], 0
    log.debug("tbsource lines '''%s''' around index %s", lines, index)
    return (lines, index)


def find_inspectable_lines(lines, pos):
    r"""Find lines in home that are inspectable.

    Walk back from the err line up to 3 lines, but don't walk back over
    changes in indent level.

    Walk forward up to 3 lines, counting \ separated lines as 1. Don't walk
    over changes in indent level (unless part of an extended line)
    """
    cnt = re.compile(r'\\[\s\n]*$')
    df = re.compile(r':[\s\n]*$')
    ind = re.compile(r'^(\s*)')
    toinspect = []
    home = lines[pos]
    home_indent = ind.match(home).groups()[0]

    before = lines[max(pos - 3, 0):pos]
    before.reverse()
    after = lines[pos + 1:min(pos + 4, len(lines))]

    for line in before:
        if ind.match(line).groups()[0] == home_indent:
            toinspect.append(line)
        else:
            break
    toinspect.reverse()
    toinspect.append(home)
    home_pos = len(toinspect) - 1
    continued = cnt.search(home)
    for line in after:
        if ((continued or ind.match(line).groups()[0] == home_indent)
                and not df.search(line)):
            toinspect.append(line)
            continued = cnt.search(line)
        else:
            break
    log.debug("Inspecting lines '''%s''' around %s", toinspect, home_pos)
    return toinspect, home_pos


class Expander:
    """Simple expression expander. Uses tokenize to find the names and
    expands any that can be looked up in the frame.
    """

    def __init__(self, locals, globals):
        self.locals = locals
        self.globals = globals
        self.lpos = None
        self.expanded_source = ''

    def __call__(self, ttype, tok, start, end, line):
        # TODO
        # deal with unicode properly

        # TODO
        # Dealing with instance members
        #   always keep the last thing seen  
        #   if the current token is a dot,
        #      get ready to getattr(lastthing, this thing) on the
        #      next call.

        if self.lpos is not None:
            if start[1] >= self.lpos:
                self.expanded_source += ' ' * (start[1] - self.lpos)
            elif start[1] < self.lpos:
                # newline, indent correctly
                self.expanded_source += ' ' * start[1]
        self.lpos = end[1]

        if ttype == tokenize.INDENT:
            pass
        elif ttype == tokenize.NAME:
            # Clean this junk up
            try:
                val = self.locals[tok]
                if callable(val):
                    val = tok
                else:
                    val = repr(val)
            except KeyError:
                try:
                    val = self.globals[tok]
                    if callable(val):
                        val = tok
                    else:
                        val = repr(val)

                except KeyError:
                    val = tok
            # FIXME... not sure how to handle things like funcs, classes
            # FIXME this is broken for some unicode strings
            self.expanded_source += val
        else:
            self.expanded_source += tok
        # if this is the end of the line and the line ends with
        # \, then tack a \ and newline onto the output
        # print line[end[1]:]
        if re.match(r'\s+\\\n', line[end[1]:]):
            self.expanded_source += ' \\\n'



import os
import unittest


class TestIssue006(unittest.TestCase):
    def test_load_nested_generator(self):
        from nose.loader import TestLoader

        where = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             'support', 'issue006'))
        l = TestLoader()
        testmod = iter(l.loadTestsFromName(where)).next()
        print
        testmod
        testmod.setUp()

        testcase = iter(testmod).next()
        expect = [
            ['tests.Test1.test_nested_generator'],
            ['tests.Test1.test_nested_generator_mult(1,)',
             'tests.Test1.test_nested_generator_mult(2,)',
             'tests.Test1.test_nested_generator_mult(3,)'],
            ['tests.Test1.test_normal_generator(1,)',
             'tests.Test1.test_normal_generator(2,)']
        ]
        for test in testcase:
            tests = map(str, test)
            print
            tests
            self.assertEqual(tests, expect.pop(0))


if __name__ == '__main__':
    unittest.main()



import os
import sys
import unittest
from nose.plugins.attrib import AttributeSelector
from nose.plugins import PluginTester

support = os.path.join(os.path.dirname(__file__), 'support')

compat_24 = sys.version_info >= (2, 4)


class AttributePluginTester(PluginTester, unittest.TestCase):
    plugins = [AttributeSelector()]
    suitepath = os.path.join(support, 'att')
    # Some cases need -a to activate and others need -A, so
    # let's treat -v as the activate argument and let individual
    # cases specify their -a arguments as part of args
    activate = '-v'

    def runTest(self):
        print('*' * 70)
        print(str(self.output))
        print('*' * 70)


class TestSimpleAttribute(AttributePluginTester):
    args = ["-a", "a"]

    def verify(self):
        assert 'test_attr.test_one ... ok' in self.output
        assert 'test_attr.test_two ... ok' in self.output
        assert 'TestClass.test_class_one ... ok' in self.output
        assert 'TestClass.test_class_two ... ok' in self.output
        assert 'TestClass.test_class_three ... ok' in self.output
        assert 'test_three' not in self.output
        assert 'test_case_two' not in self.output
        assert 'test_case_one' not in self.output
        assert 'test_case_three' not in self.output
        assert 'TestAttrClass.test_one ... ok' in self.output
        assert 'TestAttrClass.test_two ... ok' in self.output
        assert 'TestAttrClass.ends_with_test ... ok' in self.output


class TestNotSimpleAttribute(AttributePluginTester):
    args = ["-a", "!a"]

    def verify(self):
        assert 'test_attr.test_one ... ok' not in self.output
        assert 'test_attr.test_two ... ok' not in self.output
        assert 'TestClass.test_class_one ... ok' not in self.output
        assert 'TestClass.test_class_two ... ok' not in self.output
        assert 'TestClass.test_class_three ... ok' not in self.output
        assert 'test_three' in self.output
        assert 'test_case_two' in self.output
        assert 'test_case_one' in self.output
        assert 'test_case_three' in self.output


class TestAttributeValue(AttributePluginTester):
    args = ["-a", "b=2"]

    def verify(self):
        assert 'test_attr.test_one ... ok' not in self.output
        assert 'test_attr.test_two ... ok' not in self.output
        assert 'test_attr.test_three ... ok' not in self.output
        assert 'TestClass.test_class_one ... ok' not in self.output
        assert 'TestClass.test_class_two ... ok' in self.output
        assert 'TestClass.test_class_three ... ok' not in self.output
        assert 'test_case_two' in self.output
        assert 'test_case_one' in self.output
        assert 'test_case_three' in self.output


class TestAttributeArray(AttributePluginTester):
    args = ["-a", "d=2"]

    def verify(self):
        assert 'test_attr.test_one ... ok' in self.output
        assert 'test_attr.test_two ... ok' in self.output
        assert 'test_attr.test_three ... ok' not in self.output
        assert 'TestClass.test_class_one ... ok' not in self.output
        assert 'TestClass.test_class_two ... ok' not in self.output
        assert 'TestClass.test_class_three ... ok' not in self.output
        assert 'test_case_two' not in self.output
        assert 'test_case_one' not in self.output
        assert 'test_case_three' not in self.output


class TestAttributeArrayAnd(AttributePluginTester):
    args = ["-a", "d=1,d=2"]

    def verify(self):
        assert 'test_attr.test_one ... ok' in self.output
        assert 'test_attr.test_two ... ok' not in self.output
        assert 'test_attr.test_three ... ok' not in self.output
        assert 'TestClass.test_class_one ... ok' not in self.output
        assert 'TestClass.test_class_two ... ok' not in self.output
        assert 'TestClass.test_class_three ... ok' not in self.output
        assert 'test_case_two' not in self.output
        assert 'test_case_one' not in self.output
        assert 'test_case_three' not in self.output


class TestAttributeArrayOr(AttributePluginTester):
    args = ["-a", "d=1", "-a", "d=2"]

    def verify(self):
        assert 'test_attr.test_one ... ok' in self.output
        assert 'test_attr.test_two ... ok' in self.output
        assert 'test_attr.test_three ... ok' in self.output
        assert 'TestClass.test_class_one ... ok' not in self.output
        assert 'TestClass.test_class_two ... ok' not in self.output
        assert 'TestClass.test_class_three ... ok' not in self.output
        assert 'test_case_two' not in self.output
        assert 'test_case_one' not in self.output
        assert 'test_case_three' not in self.output


class TestInheritance(AttributePluginTester):
    # Issue #412
    args = ["-a", "from_super"]

    def verify(self):
        assert 'TestSubclass.test_method ... ok' in self.output
        assert 'TestAttrSubClass.test_sub_three ... ok' in self.output
        assert 'TestAttrSubClass.test_one ... ok' in self.output
        assert 'TestAttrSubClass.added_later_test ... ok' in self.output
        assert 'test_two' not in self.output
        assert 'test_case_one' not in self.output
        assert 'test_case_three' not in self.output


class TestStatic(AttributePluginTester):
    # Issue #411
    args = ["-a", "with_static"]
    suitepath = os.path.join(support, 'att', 'test_attr.py:Static')

    def verify(self):
        assert 'Static.test_with_static ... ok' in self.output
        assert 'test_case_two' not in self.output
        assert 'test_case_one' not in self.output
        assert 'test_case_three' not in self.output


class TestClassAndMethodAttrs(AttributePluginTester):
    # Issue #324
    args = ["-a", "meth_attr=method,cls_attr=class"]

    def verify(self):
        assert '(test_attr.TestClassAndMethodAttrs) ... ok' in self.output
        assert 'test_case_two' not in self.output
        assert 'test_case_one' not in self.output
        assert 'test_case_three' not in self.output


# Issue #771
class TestTopLevelNotSelected(AttributePluginTester):
    suitepath = os.path.join(support, 'issue771')
    args = ["-a", "!a"]

    def verify(self):
        # Note: a failure here may mean that the test case selection is broken
        # rather than the attribute plugin, but the issue more easily manifests
        # itself when using attributes.
        assert 'test.test_b ... ok' in self.output
        assert 'test_a (test.TestBase) ... ok' in self.output
        assert 'TestDerived' not in self.output


# Issue #728
class TestStaticMethod(AttributePluginTester):
    suitepath = os.path.join(support, 'attrib-static')
    args = ["-a", "!slow"]

    def verify(self):
        assert 'test.TestAttrib.test_static ... ok' in self.output
        assert 'Ran 1 test' in self.output


if compat_24:
    class TestAttributeEval(AttributePluginTester):
        args = ["-A", "c>20"]

        def verify(self):
            assert 'test_attr.test_one ... ok' not in self.output
            assert 'test_attr.test_two ... ok' not in self.output
            assert 'test_attr.test_three ... ok' not in self.output
            assert 'TestClass.test_class_one ... ok' not in self.output
            assert 'TestClass.test_class_two ... ok' not in self.output
            assert 'TestClass.test_class_three ... ok' not in self.output
            assert 'test_case_two' in self.output
            assert 'test_case_one' not in self.output
            assert 'test_case_three' not in self.output

# Avoid trying to run base class as tests
del AttributePluginTester

if __name__ == '__main__':
    # import logging
    # logging.basicConfig(level=logging.DEBUG)
    unittest.main()



"""
Module-level doctest

    >>> 1
    1
    >>> add_one(_)
    2
"""


def add_one(i):
    """
    function doctest

    >>> add_one(1)
    2
    >>> add_one(2)
    3
    >>> add_one('steve')
    joe    
    """
    return i + 1



import pickle
import sys
import unittest

from nose import case
from nose.config import Config
from nose.loader import TestLoader
from nose.plugins import multiprocess
from nose.plugins.skip import SkipTest

try:
    # 2.7+
    from unittest.runner import _WritelnDecorator
except ImportError:
    from unittest import _WritelnDecorator


class ArgChecker:
    def __init__(self, target, args):
        self.target = target
        self.args = args
        # skip the id and queues
        pargs = args[7:]
        self.pickled = pickle.dumps(pargs)
        try:
            testQueue = args[1]
            testQueue.get(timeout=0)
        except:
            pass  # ok if queue is empty

    def start(self, *args):
        pass

    def is_alive(self):
        return False


def setup(mod):
    multiprocess._import_mp()
    if not multiprocess.Process:
        raise SkipTest("multiprocessing not available")
    mod.Process = multiprocess.Process
    multiprocess.Process = ArgChecker


class T(unittest.TestCase):
    __test__ = False

    def runTest(self):
        pass


def test_mp_process_args_pickleable():
    # TODO(Kumar) this test needs to be more succint.
    # If you start seeing it timeout then perhaps we need to skip it again.
    # raise SkipTest('this currently gets stuck in poll() 90% of the time')
    test = case.Test(T('runTest'))
    config = Config()
    config.multiprocess_workers = 2
    config.multiprocess_timeout = 5
    runner = multiprocess.NoseMultiProcessTestRunner(
        stream=_WritelnDecorator(sys.stdout),
        verbosity=10,
        loaderClass=TestLoader,
        config=config)
    runner.run(test)



import unittest


class TestIssue230(unittest.TestCase):

    def test_generator_yield_value(self):
        from nose.loader import TestLoader

        def test():
            pass

        def gen():
            yield test

        loader = TestLoader()
        suite = loader.loadTestsFromGenerator(gen, module=None)
        testcase = iter(suite).next()
        self.assertEqual(testcase.test.test, test)


if __name__ == '__main__':
    unittest.main()



import unittest

from nose import case
from nose.config import Config
from nose.plugins import Plugin, PluginManager


class TestTestCasePluginCalls(unittest.TestCase):

    def test_describe_test_called(self):
        class Descrip(Plugin):
            counter = 0
            enabled = True

            def describeTest(self, test):
                return "test #%s" % id(test)

            def testName(self, test):
                self.counter += 1
                return "(%s) test" % self.counter

        class TC(unittest.TestCase):
            def test_one(self):
                pass

            def test_two(self):
                pass

        config = Config(plugins=PluginManager(plugins=[Descrip()]))

        c1 = case.Test(TC('test_one'), config=config)
        c2 = case.Test(TC('test_two'), config=config)

        self.assertEqual(str(c1), '(1) test')
        self.assertEqual(str(c2), '(2) test')
        assert c1.shortDescription().startswith('test #'), \
            "Unexpected shortDescription: %s" % c1.shortDescription()
        assert c2.shortDescription().startswith('test #'), \
            "Unexpected shortDescription: %s" % c2.shortDescription()


if __name__ == '__main__':
    unittest.main()
